/****************************************************************/
/*                                                              */
/*    << SECS/HSMS Driver Program For Windows System >>         */
/*              DSDR Series                                     */
/*                                                              */
/*              file name : dsh_u.h                             */
/*                                                              */
/*  Produced by Data Map Corporation in 2009.                   */
/*  All rights reserved. No part of this program may be copied  */
/*  in anyway, or by any means, without permission from         */
/*  Data Map Corporation.                                       */
/*                                                              */
/****************************************************************/
#ifndef DSH_Header              //----------------> Header Start

/*------+-------+-------+-------+-------+-------+-------+-------+------*/
/* Data Type Macro                                                     */
/*------+-------+-------+-------+-------+-------+-------+-------+------*/
#ifndef DSH_TYPES_DEF
    #define TYPES_DEF
    typedef char    CHAR;
    typedef unsigned char UCHAR;
    typedef unsigned char BYTE;
    typedef short   SHORT;
    typedef unsigned short USHORT;
    typedef unsigned int UINT;
    typedef int     INT;
    typedef int     ER;
    typedef long    LONG;
    typedef int     ID_PT;
    typedef int     ID_DV;
    typedef int     ID_TR;
    typedef unsigned long ULONG;
#ifndef NIL
    #define NIL     (0xffffffff)
#endif
#endif

/*------+-------+-------+-------+-------+-------+-------+-------+------*/

#define AVAILABLE_TRID          1                       // for GetDebugInfo
#define AVAILABLE_MEMORY        2                       //  "

/*------+-------+-------+-------+-------+-------+-------+-------+------*/
/* Item Data Code( SECSII)                                             */
/*------+-------+-------+-------+-------+-------+-------+-------+------*/
#define ICODE_L         (0x00>>2)               // List
#define ICODE_A         (0x40>>2)               // A
#define ICODE_J         (0x44>>2)               // J
#define ICODE_B         (0x20>>2)               // Binary
#define ICODE_I1        (0x64>>2)               // I1
#define ICODE_I2        (0x68>>2)               // I2
#define ICODE_I4        (0x70>>2)               // I3
#define ICODE_I8        (0x60>>2)               // I4
#define ICODE_U1        (0xa4>>2)               // U1
#define ICODE_U2        (0xa8>>2)               // U2
#define ICODE_U4        (0xb0>>2)               // U4
#define ICODE_U8        (0xa0>>2)               // U8
#define ICODE_F4        (0x90>>2)               // F4
#define ICODE_F8        (0x80>>2)               // F8
#define ICODE_BOOLEAN   (0x24>>2)               // BOOL
#define ICODE_END       (0xf4>>2)               // END

/*------+-------+-------+-------+-------+-------+-------+-------+------*/
/* DSH function Return Value Macro                                     */
/*------+-------+-------+-------+-------+-------+-------+-------+------*/
#define EI_NORMAL                   0       // Normal End
#define EI_POLL_TRUE                1       // Receive Data Available
#define EI_SEND_OK                  2       // Send Normal End(wbit=1)
#define EI_SEND_W2_OK               3       // Send Normal End(wbit=2)
#define EI_ERROR                    (-1)    // Error End
#define EI_DVR_NOT_OPENED           (-2)    // Driver Not Opened
#define EI_DVR_ALREADY_OPENED       (-3)    // Driver Already Opened
#define EI_DEVICE_NOT_OPENED        (-4)    // Device Not Opened
#define EI_DEVICE_ALREADY_OPENED    (-5)    // Device Already Opened
#define EI_DEVICE_UNDEFINED         (-6)    // Device Undefined
#define EI_PORT_NOT_OPENED          (-7)    // Port Not Opened
#define EI_PORT_ALREADY_OPENED      (-8)    // Port Already Opened
#define EI_PORT_UNDEFINED           (-9)    // Port Undefined
#define EI_POLL_FALSE               (-10)   // No Event Available
#define EI_SEND_FAIL                (-11)   // Send Fail
#define EI_TRID_BUSY                (-12)   // Transaction Busy
#define EI_TRID_NOT_FOUND           (-13)   // TransactionID Not Found
#define EI_TR_TIMEOUT               (-14)   // Transaction Timeout
#define EI_ABORT                    (-15)   // Transaction aborted
#define EI_DATA_ITEM_ERR            (-16)   // Data Item Error
#define EI_TYPE_ERR                 (-17)   // Data type Error
#define EI_ITEM_POS_ERR             (-18)   // position Error
#define EI_ARRAY_SIZE_ERR           (-19)   // Buff Size was Short
#define EI_MSG_FORMAT_ERR           (-20)   // Message Format Error
#define EI_MSG_SIZE_ERR             (-21)   // SECS Msg Length Error
#define EI_SEND_W2_FAIL             (-22)   // Send Error( wbit=2)
#define EI_NOT_READY                  1     // not selected

#define TCPIP_NOT_CONNECTED             0       // HSMS not connected
#define HSMS_NOT_SELECTED               1       // HSMS not selected
#define HSMS_SELECTED                   2       // HSMS selected

/*------+-------+-------+-------+-------+-------+-------+-------+------*/
/* SECS Msg Header \                                              */
/*------+-------+-------+-------+-------+-------+-------+-------+------*/
typedef struct{
        UCHAR       dvid[2];
    UCHAR       stream;
    UCHAR       function;
    UCHAR       blkid[2];
    UCHAR       sybt[4];
    } SecsHeader, *PSecsHeader;         // SECSүން̍\

/*------+-------+-------+-------+-------+-------+-------+-------+------*/
/* SECS Msg Information \                                         */
/*------+-------+-------+-------+-------+-------+-------+-------+------*/
typedef struct{
    UINT    stream;             // stream id
    UINT    function;           // function id
    UINT    wbit;               // wait bit
    int     length;             // msg length
    UCHAR   *buffer;            // msg buffer
    int     error;              // error
    int     next;               // next item
    UCHAR   *txtp;              // (p)
    int     txtc;               // (   "      )
    int     work[2];            // (p)
    } DSHMSG, *PDSHMSG;
/*------+-------+-------+-------+-------+-------+-------+-------+------*/
/* |[gǉ` ( D_AppendPort() )                                */
/*------+-------+-------+-------+-------+-------+-------+-------+------*/
#define DSH_P_SECS              0               // Protocol = SECS
#define DSH_P_HSMS              1               //          = HSMS
#define DSH_P_EQUPMENT          0               // op" mode = EQUIPMENT
#define DSH_P_HOST              1               //   "      = HOST
#define DSH_P_ACTIVE            0               // entity t = ACTIVE
#define DSH_P_PASSIVE           1               //   "      = PASSSIVE

/*------+-------+-------+-------+-------+-------+-------+-------+------*/
/* DSHDR2 Function Prototype                                           */
/*------+-------+-------+-------+-------+-------+-------+-------+------*/
INT  D_StartDriver( CHAR *ConfigFile );
INT  D_StopDriver( void );
INT  D_StartPort( ID_PT ptno );
INT  D_StopPort( ID_PT ptno );
INT  D_StartDevice( ID_DV dvno );
INT  D_StopDevice( ID_DV dvno );
INT  D_SendRequest( ID_DV dvno, DSHMSG *msg, ID_TR *tridptr );
INT  D_SendResponse( DSHMSG *msg, INT trid );
INT  D_Receive( ID_TR trid, DSHMSG *msg );
INT  D_PollPort( ID_PT ptno, ID_DV *dvno, DSHMSG *msg, ID_TR *trid );
INT  D_PollDevice( ID_DV dvno, DSHMSG *msg, ID_TR *trid );
INT  D_SetPortEvent( INT ptno, INT Evt );
INT  D_SetDeviceEvent( INT dvno, INT Evt );
INT  D_FreeTrid( ID_TR trid );
INT  D_CheckTridBusy( ID_TR trid );

INT  D_InitItemGet( DSHMSG *msg );
INT  D_GetItem( DSHMSG *msg, INT icode, void *data, INT size );
INT  D_GetItemByPos( DSHMSG *msg, INT pos, INT icode, void *data, INT size );
INT  D_InitItemPut( DSHMSG *msg );
INT  D_PutItem( DSHMSG *msg, INT icode, void *data, INT size );
INT  D_PutItemByPos( DSHMSG *msg, INT pos, INT icode, void *data, INT size );
INT  D_GetItemCodeSize( INT icode );
INT  D_GetItemSize( DSHMSG *msg, INT icode, INT maxcount );

INT  D_CheckReady( ID_DV dvno );

int     D_CreateEvent( void );
void    D_CloseEvent( int evtid );
int     D_WaitEvent( int evtid, int tms );
int     D_PostEvent( int evtid );
void    D_ResetEvent( int evtid );
void    D_msleep( int tms );
void    DSH_printf( const char * pfmt, ... );

/*--------------------------------------------------------------------*/
INT  D_GetDebugInfo( INT item_info, void *value );

void  D_SetLogFile( char *NewFileName, char *OldFileName );

void  D_GetSerialNo( char *buff );

#define DSH_Header
#endif               //<---------------- Header End

/*------ End Of File ---+-------+-------+-------+-------+-------+------*/
