/*--------------------------------------------------------------------------*/
/*                                                                          */
/*  DshEng4   - GEM, GEM300 Equipment Communication Control Package         */
/*                                                                          */
/*    enginfo.h - ENG4 Information Definition (Structure, constant)         */
/*                                                                          */
/*  Copyright (c)2009 by Data Map Corp.                                     */
/*                                                                          */
/*--------------------------------------------------------------------------*/

/*---------------------------------------------------------------------*/

/*===== CE - collection event =========================================*/
/*---------------------------------------------------------------------*/
/* CE - Collection Event                                               */
/*---------------------------------------------------------------------*/
typedef struct{
        TCEID   ceid;                   // ceid
        char    *name;                  // ce name
        int     ceed;                   // enable=1, disable=0
        int     rp_count;               // link Ăreport ID
        char    **rpname;               // rp name list
        TRPID   *rpid;                  // report ID list
        } TCE_INFO;


/*---------------------------------------------------------------------*/
/* Variable Content                                                    */
/*---------------------------------------------------------------------*/
typedef struct{
        TVID        vid;
        int         format;
        int         asize;
        void        *dptr;                  // fmt LȂnesting ->TV_CONTENT
        } TV_CONTENT;

/*---------------------------------------------------------------------*/
/* Report Content                                                      */
/*---------------------------------------------------------------------*/
typedef struct{
        TRPID       rpid;
        int         v_count;
        TV_CONTENT  **v_list;
        } TRP_CONTENT;

/*---------------------------------------------------------------------*/
/* Collection Event Content                                            */
/*---------------------------------------------------------------------*/
typedef struct{
        TCEID       ceid;
        int         rp_count;
        TRP_CONTENT **rp_list;
        int         next_rp;
        int         next_v;
        } TCE_CONTENT;

/*---------------------------------------------------------------------*/
/*  for S2F33 Define Report                                            */
/*---------------------------------------------------------------------*/
typedef struct{
        TRPID       rpid;
        int         count;
        TVID        *vid_list;
        } TRP_DEFINE;

/*---------------------------------------------------------------------*/
typedef struct{
        int         count;
        TRP_DEFINE  **rp_list;
        } TRP_LIST;
/*---------------------------------------------------------------------*/
/*  for S2F35 Link Event Report                                        */
/*---------------------------------------------------------------------*/
typedef struct{
        TCEID       ceid;
        int         count;
        TRPID       *rpid_list;
        } TCE_LINK;

/*---------------------------------------------------------------------*/
typedef struct{
        int         count;
        TCE_LINK    **ce_list;
        } TCE_LIST;

/*===== Report ( for CE ) =============================================*/

#define RP_ATTR_V   0                   // V(f[^ϐ)
#define RP_ATTR_SV  1                   // SV(ԕϐ)
#define RP_ATTR_EC  2                   // EC(u萔)

/*---------------------------------------------------------------------*/
/* RP - Report                                                         */
/*---------------------------------------------------------------------*/
typedef struct{
        TRPID   rpid;                   // rpid
        char    *name;                  // report name
        int     v_count;                // vid list count ( link )
        char    **vname;                // variable name list
        TDVID   *vid;                   // vid list
        int     *attr;                  // vid̎ގ(EC=0, SV=1, V=2)
        } TRP_INFO;

/*===== Variable Limit ================================================*/
/*---------------------------------------------------------------------*/
/* VLIMIT - Variable data limit id information                         */
/*---------------------------------------------------------------------*/
typedef struct{
        int         limit_id;           // VLIMITID
        void        *upperdb;           // upper limit data
        void        *lowerdb;           // lower limit data
        } TVLIMIT_ID_INFO;

/*---------------------------------------------------------------------*/
/* VLimit Information(VID)                                             */
/*---------------------------------------------------------------------*/
typedef struct{
        TDVID        vid;               // vid
        int         limit_count;        // # of limit id
        TVLIMIT_ID_INFO **limitid_list; // limit information list
        } TVLIMIT_INFO;

/*---------------------------------------------------------------------*/
typedef struct{
        int          vid_count;         // # of variable
        TVLIMIT_INFO **limit_list;      // v limit information list
        } TVLIMIT_LIST;

/*---------------------------------------------------------------------*/
/* VID LIMIT INFORMATION                                               */
/*---------------------------------------------------------------------*/
typedef struct{
        TLIMITID    limit_id;
        void        *upperdb;
        void        *lowerdb;
} TLIMIT_ID_INFO;

/*---------------------------------------------------------------------*/
/* Limit Information(VID)                                              */
/*---------------------------------------------------------------------*/
typedef struct{
        TDVID        vid;
        int         limit_count;
        TLIMIT_ID_INFO **limitid_list;
        int         format;             // for upperdb & lowerdb
        int         asize;              //   "         "   "
} TLIMIT_INFO;

/*---------------------------------------------------------------------*/
/* Limit List (S2F45)                                                  */
/*---------------------------------------------------------------------*/
typedef struct{
        int         vid_count;
        TLIMIT_INFO **limit_list;
} TLIMIT_LIST;

/*---------------------------------------------------------------------*/
/* TLIMIT_ERR_INFO                                                     */
/*---------------------------------------------------------------------*/
typedef struct{
        TDVID       vid;
        int         lvack;
        int         lmt_count;
        TLIMITID    *limit_id;
        int         *limitack;           // B
} TLIMIT_ERR_INFO;

/*---------------------------------------------------------------------*/
/* TLIMIT_ERR_LIST                                                     */
/*---------------------------------------------------------------------*/
typedef struct{
        int         vlaack;
        int         err_count;
        TLIMIT_ERR_INFO **limit_list;
} TLIMIT_ERR_LIST;

/*---------------------------------------------------------------------*/
/* Limit Information(VID) - S2F48                                      */
/*---------------------------------------------------------------------*/
typedef struct{
        TVID        vid;
        char        *units;
        int         format;             // for upperdb & lowerdb
        int         asize;              //   "         "   "
        void        *limit_min;
        void        *limit_max;
        int         limit_count;
        TLIMIT_ID_INFO **limitid_list;
} TLIMIT_RSP_INFO;

/*---------------------------------------------------------------------*/
/* Limit List (S2F48)                                                  */
/*---------------------------------------------------------------------*/
typedef struct{
        int         vid_count;
        TLIMIT_RSP_INFO **limit_list;
} TLIMIT_RSP_LIST;

/*===== EC, SV, DVVAL Variable information ============================*/
//---- lmt_state
#define ST_LIMIT_RESET          0                       // RESET
#define ST_LIMIT_IN_RANGE       1                       // ̈ limitid = no change
#define ST_LIMIT_OUT_RANGE      2                       // ̈O limitid = out of range
//---- lmt_dir
#define DIR_LIMIT_UP            0                       // UPPER֕ω
#define DIR_LIMIT_DOWN          1                       // LOWER֕ω
/*---------------------------------------------------------------------*/
/* Limit Event Info   DSHGEM-LIB --> EQ APP                            */
/*---------------------------------------------------------------------*/
typedef struct{
        TVID    vid;                    // vid
        char    *value;                 // value in ascii
        int     limitid;                // limit id
        int     dir;                    // transient direction (0=up, 1=down)
}TVLIMIT_EVENT_INFO;

/*---------------------------------------------------------------------*/
/* DVVAL - Data Variable Value                                         */
/*---------------------------------------------------------------------*/
typedef struct{
        TDVID   vid;
        char    *name;
        BYTE    format;                 // Format A,B,I1.....
        int     asize;
        int     min_asize;
        int     max_asize;
        void    *val;
        void    *min;                   // low limit
        void    *max;                   // high  "
        void    *nominal;               // default value
        char    *units;
        int     lmt_state;              // 0=sA1=mA2=̈Jڂ
        int     c_limitid;              // LIMITID
        int     lmt_dir;                // 0=  1=
        int     limitid_count;
        TVLIMIT_ID_INFO **limitid_list;
        int     ce_count;
        char    **ce_name;
        TCEID   *ceid;
        int     count;
        void    **valx;
        char    **capt;
//        void    *old_min;
//        void    *old_max;
//        SYSTEMTIME  tstamp;             // time stamp  2008.2.18
        } TV_INFO;

/*---------------------------------------------------------------------*/
/* EC - Equipment Constant Information                                 */
/*---------------------------------------------------------------------*/
typedef struct{
        TECID   ecid;
        char    *name;
        int     format;
        int     asize;
        int     min_asize;
        int     max_asize;
        void    *val;                   // value
        void    *min;                   // low limit
        void    *max;                   // high  "
        void    *nominal;               // default value
        char    *units;
        int     lmt_state;              // 0=sA1=mA2=̈Jڂ
        int     c_limitid;              // LIMITID
        int     lmt_dir;                // 0=  1=
        int     limitid_count;
        TVLIMIT_ID_INFO **limitid_list;
        int     ce_count;
        char    **ce_name;
        TCEID   *ceid;
        int     count;
        void    **valx;
        char    **capt;
//        void    *old_min;
//        void    *old_max;
//        SYSTEMTIME  tstamp;             // time stamp  2008.2.18
        } TEC_INFO;

/*---------------------------------------------------------------------*/
/* SV - Status Variable Information                                    */
/*---------------------------------------------------------------------*/
typedef struct{
        TSVID   svid;
        char    *name;
        int     format;
        int     asize;
        int     min_asize;
        int     max_asize;
        void    *val;                   // l
        void    *min;                   // low limit
        void    *max;                   // high  "
        void    *nominal;               // default value
        char    *units;
        int     lmt_state;              // 0=sA1=mA2=̈Jڂ
        int     c_limitid;              // LIMITID
        int     lmt_dir;                // 0=  1=
        int     limitid_count;
        TVLIMIT_ID_INFO **limitid_list;
        int     ce_count;
        char    **ce_name;
        TCEID   *ceid;
        int     count;
        void    **valx;
        char    **capt;
//        void    *old_min;
//        void    *old_max;
//        SYSTEMTIME  tstamp;             // time stamp  2008.2.18
        } TSV_INFO;

/*---------------------------------------------------------------------*/
typedef struct{
        TVID        vid;                // vid
        int         format;             // format A, B, I, I2...
        int         asize;              // value array size
        void        *value;             // value
        } TV_VALUE;

/*---------------------------------------------------------------------*/
typedef struct{
        int         count;              // # of var value list
        TV_VALUE    **vv_list;          // var value list
        } TV_VALUE_LIST;

/*---------------------------------------------------------------------*/
typedef struct{
        TSVID       svid;               // svid
        char        *name;              // sv name
        char        *units;             // unit name
        } TSV_NAME;

/*---------------------------------------------------------------------*/
typedef struct{
        int         count;              // # of sv name
        TSV_NAME    **name_list;        // sv name list
        } TSV_NAME_LIST;

/*---------------------------------------------------------------------*/
typedef struct{
        TECID       ecid;               // ecid
        char        *name;              // ec name
        int         format;             // ec format
        int         asize;              // ec value array size
        void        *ecmin;             // min value
        void        *ecmax;             // max value
        void        *ecdef;             // nominal value
        char        *units;             // unit name
        } TEC_NAME;

/*---------------------------------------------------------------------*/
typedef struct{
        int         count;              // # of ec name
        TEC_NAME    **name_list;        // ec name list
        } TEC_NAME_LIST;


/*===== SV Trace information ==========================================*/
/*---------------------------------------------------------------------*/
/* Trace ֘A - S2F23, S6F1                                        */
/*---------------------------------------------------------------------*/
typedef struct{
        int     index;                  // info index
        int     state;                  // state
        char    *name;                  // trace name
        char    *trid;                  // trace id
        int     format;                 // trace id format
        int     asize;                  // trace id array size
        int     max_asize;
        char    *dsper;                 // traceԎ
        int     dsper_time;             // traceԎ-l
        int     totsmp;                 // total sample
        int     tot_fmt;
        int     tot_asize;
        int     repgsz;                 // report group size
        int     gsz_fmt;
        int     gsz_asize;
        int     svid_count;             // svid listsize
        TSVID   *svid_list;             // svid list
        } TTRACE_INFO;

/*---------------------------------------------------------------------*/
/* Trace Data by S6F1                                                  */
/*---------------------------------------------------------------------*/
typedef struct{
        int         format;             // sv value format
        int         asize;              // value array size
        void        *sv;                // sv value
        } TTRACE_SV;

/*---------------------------------------------------------------------*/
typedef struct{
        void        *trid;              // trace id
        int         format;             // trace id format
        int         asize;              // trace id array size
        int         smpln;              // sampling no.
        char        *stime;             // start time
        int         count;              // no. of data
        TTRACE_SV   **sv_list;          // sv ptr list
        } TTRACE_DATA;

/*===== ALARM information =============================================*/
/*---------------------------------------------------------------------*/
/* ALARM - Alarm Inforamtion                                           */
/*---------------------------------------------------------------------*/
typedef struct{
        TALID   alid;                   // alid
        char    *name;                  // alname
        TALCD   alcd;                   // alcd
        char    *altx;                  // altx
        int     aled;                   // enable / disable
        char    *ceon_name;             // ceon name
        TCEID   ceid_on;                // ceon id
        char    *ceoff_name;            // ceoff name
        TCEID   ceid_off;               // ceoff id
        } TAL_INFO;

typedef struct{
        int         on_off;             // enable / disable flag
        TALID       alid;               // alid
        TALCD       alcd;               // alcd
        char        *altx;              // altx
        } TAL_S5F1_INFO;

typedef struct{
        int         count;              // # of alarm infomation
        TAL_S5F1_INFO    **al_list;     // alarm information list
        } TAL_S5F6_LIST;

/*===== SPOOL Infromation =============================================*/
/*---------------------------------------------------------------------*/
/* Spoolstreamfunction                                     */
/*---------------------------------------------------------------------*/
typedef struct{                         // S2F43̎擾
        int     stream;                 // stream
        int     f_count;                // # of functions
        int     *func_list;             // fuction list
        } TSTRE_INFO;

/*---------------------------------------------------------------------*/
/* Spoolstream  S2F43쐬                                  */
/*---------------------------------------------------------------------*/
typedef struct{
        int         s_count;            // # of streams
        TSTRE_INFO  **stre_list;        // stream info list
        } TSPOOL_INFO;

/*---------------------------------------------------------------------*/
/* Spoolstream, Function Error                                 */
/*---------------------------------------------------------------------*/
typedef struct{                         // S2F43̎擾
        int     strack;                 // ack for stream
        int     stream;                 // stream
        int     f_count;                // # of functions
        int     *func_list;             // fuction list
        int     *func_err;              // func err( 0/1 )
        } TSTRE_ERR_INFO;

/*---------------------------------------------------------------------*/
/* Spoolstream Error                                           */
/*---------------------------------------------------------------------*/
typedef struct{
        int         rsack;              // ack for s2f43
        int         err_count;          // # of streams
        TSTRE_ERR_INFO  **stre_list;    // err stream info list
        } TSPOOL_ERR_INFO;

/*---------------------------------------------------------------------*/
/* SpoolǗ                                                       */
/*---------------------------------------------------------------------*/
typedef struct{
        int     f_count;                // # of function
        int     *func_list;             // function code list
        } TSPOOL_TAB;


/*===== PP Process Program ============================================*/
/*---------------------------------------------------------------------*/
/* Process Program Information                                         */
/*---------------------------------------------------------------------*/
typedef struct{
        int         index;              // info index
        int         state;              // state
        char        *name;              // pp name
        int         ppid_fmt;           // ppid format
        int         ppid_size;          // ppid size
        int         max_ppid_size;      // max ppid size
        void        *ppid;              // ppid
        int         ppbody_fmt;         // ppbody format
        int         ppbody_size;        // ppbody size
        int         max_ppbody_size;    // max ppbody size
        void        *ppbody;            // ppbody value
        } TPP_INFO;

/*---------------------------------------------------------------------*/
/* S7F3 : Remote Command Information Record                           */
/*---------------------------------------------------------------------*/
typedef struct{
        char        *ppid;              // ppid
        int         ppbody_fmt;         // ppbody item fmt
        int         ppbody_size;        // ppbody data array size
        void        *ppbody;            // ppbody
        } TPP_REQ_INFO;

/*---------------------------------------------------------------------*/
/* S7F1                                                                */
/*---------------------------------------------------------------------*/
typedef struct{
        int         ppid_fmt;           // ppid item fmt
        int         ppid_size;          // ppid data array size
        void        *ppid;              // ppid
        int         length;             // length
        } TPPINQ_INFO;

/*---------------------------------------------------------------------*/
/* S7F20 Current PPED Data                                             */
/*---------------------------------------------------------------------*/
typedef struct{
        int         count;              // # of ppid
        char        **ppid_list;        // ppid list
        } TPPID_LIST;

/*---------------------------------------------------------------------*/
/* S7F27 - Process Program Verification Send                           */
/*---------------------------------------------------------------------*/
typedef struct{
        int         ackc7a;             // ack
        int         seqnum;             // sequence number
        char        *errw7;             // err text
        } TPP_PVS_INFO;

typedef struct{
        char        *ppid;              // ppid
        int         err_count;          // # of err
        TPP_PVS_INFO **err_list;        // error information list
        } TPP_PVS_LIST;

/*===== FPP Formatted Process Program =================================*/
/*---------------------------------------------------------------------*/
/* Formatted Program Parameter                                         */
/*---------------------------------------------------------------------*/
typedef struct{
        int         ppara_fmt;          // para format
        int         ppara_size;         // para value array size
        int         max_ppara_size;     // max value para array size
        void        *ppara;             // parameter value
        } TFPP_PARA;

/*---------------------------------------------------------------------*/
/* Process Commmand Information                                        */
/*---------------------------------------------------------------------*/
typedef struct{
        int         ccode_fmt;          // command code
        int         ccode_size;         // command code array size
        int         max_ccode_size;     // max command code array size
        void        *ccode;             // command code
        int         ppara_count;        // # of para count
        TFPP_PARA   **ppara_list;       // parameter list
        } TFPP_CCODE;
/*---------------------------------------------------------------------*/
/* Formatted Process Program Information                               */
/*---------------------------------------------------------------------*/
typedef struct{
        int         index;              // info index
        int         state;              // state
        char        *name;              // fpp name
        int         ppid_fmt;           // fppid format
        int         ppid_size;          // fppid size
        int         max_ppid_size;      // max fppid size
        char        *ppid;              // fppid
        char        *mdln;              // model name
        char        *softrev;           // soft revision
        int         ccode_count;        // # of process commands
        TFPP_CCODE  **ccode_list;       // command list
        } TFPP_INFO;                     // formatted process program

/*===== Carrier Information ===========================================*/
/*---------------------------------------------------------------------*/
/* Carrier Slot Information                                            */
/*---------------------------------------------------------------------*/
typedef struct{                         // slot information
        int         status;             // status
        int         slotid;             // slotid (U1)
        char        *mid;               // mid
        char        *substid;           // subst id
        char        *substloc;          // subst loc
        } TSLOT_INFO;

/*----- Carrier Usage -------------------------------------------------*/
#define CAR_PRODUCT             0
#define CAR_TEST                1
#define CAR_DUMMY               2
#define CAR_FILLER              3

/*----- Carrier ID Status ---------------------------------------------*/
#define CAR_ID_NOT_READ         0
#define CAR_ID_READ             1
#define CAR_ID_VER_OK           2       // Verification
#define CAR_ID_VER_FAIL         3

/*----- Carrier Accessing Status --------------------------------------*/
#define CAR_NOT_ACCESSED        0
#define CAR_IN_ACCESS           1
#define CAR_COMPLETE            2
#define CAR_STOPPED             3

/*----- Carrier SlotMap Status ----------------------------------------*/
#define CAR_SMAP_NOTREAD        0
#define CAR_SMAP_READ           1
#define CAR_SMAP_VER_OK         2
#define CAR_SMAP_VER_FAILED     3

/*---------------------------------------------------------------------*/
/* Carrier Information                                                 */
/*---------------------------------------------------------------------*/
typedef struct{
        int         index;              // info index
        int         state;              // state
        int         capacity;           // capacity
        char        *usage;             // usage
        char        *carid;             // carid
        int         map_status;         // map status
        int         id_status;          // id status
        int         acc_status;         // acc status
        char        *location;          // location
        int         slot_count;         // # of slot
        TSLOT_INFO  **slot_list;        // slot infomation list
        } TCAR_INFO;

/*===== Substrate Information =========================================*/
typedef struct{                         // Location history
        int         hist_count;         // # of history
        char        **subst_locID;      // location id list
        char        **TimeIn;           // Time in list
        char        **TimeOut;          // Time out list
        } TSUBST_LOC_HIST;

/*---------------------------------------------------------------------*/
/* Substrate Information                                               */
/*---------------------------------------------------------------------*/
typedef struct{
        int         index;              // info index
        int         state;              // infdo state
        char        *substid;           // substid
        char        *acquiredID;        // acuired id
        char        *lotID;             // lot id
        char        *subst_locID;       // location id
        char        *subst_source;      // source loc id
        char        *subst_destination; // destination loc id
        char        *batch_locID;       // batch loc id
        char        *subst_pos_in_batch;// position in batch
        int         substIDstatus;      // id status
        int         material_status;    // material status
        int         subst_proc_state;   // processing status
        int         subst_state;        // subst state
        int         subst_loc_state;    // loc state
        int         subst_usage;        // usage
        int         subst_type;         // type
        TSUBST_LOC_HIST *hist;          // location history list
        } TSUBST_INFO;

/*===== Object Error Information ======================================*/

//----- ERRCODE ------------------------------------------------------------
#define ERC_OK                          0
#define ERC_OBJ_ERROR                   1
#define ERC_TARGET_OBJ_ERROR            2
#define ERC_OBJ_INSTANCE_ERROR          3
#define ERC_OBJ_ATTR_ERROR              4
#define ERC_OBJ_RO_ERROR                5               // Read only
#define ERC_OBJ_TYPE_ERROR              6
#define ERC_OBJ_ATTRVAL_ERROR           7
#define ERC_SYNTAX_ERROR                8
#define ERC_OBJ_CHECK_ERROR             9
#define ERC_INVALID                     10
#define ERC_OBJ_BUSY_ERROR              11

/*---------------------------------------------------------------------*/
/* TERR_INFO                                                           */
/*---------------------------------------------------------------------*/
typedef struct{
        int     errcode;                // error code
        char    *errtext;               // error text
        } TERR_INFO;


/*===== Carrier Action Control (S3F17) ================================*/
// Carrier Action Index
#define CA_Bind                         0
#define CA_CancelBind                   1
#define CA_CancelAllCarrierOut          2
#define CA_CancelCarrier                3
#define CA_CancelCarrierAtPort          4
#define CA_CancelCarrierNotification    5
#define CA_CancelCarrierOut             6
#define CA_CarrierIn                    7
#define CA_CarrierNotification          8
#define CA_CarrierOut                   9
#define CA_CarrierReCreate              10
#define CA_CarrierRelease               11
#define CA_ProceedWithCarrier           12

/*---------------------------------------------------------------------*/
// Carrier Action Parameter Attr Index
#define CA_ObjId                        0
#define CA_Capacity                     1
#define CA_CarrierAccessingStatus       2
#define CA_CarrierIDStatus              3
#define CA_ContentMap                   4
#define CA_LocationID                   5
#define CA_SlotMap                      6
#define CA_SlotMapStatus                7
#define CA_SubStrateCount               8
#define CA_Usage                        9

/*---------------------------------------------------------------------*/
/* TCACT_ERR_INFO                                                      */
/*---------------------------------------------------------------------*/
typedef struct{                         // s3f18 information
        int         caack;              // caack for s3f18
        int         err_count;          // # no error
        TERR_INFO   **err_list;         // error list
        } TCACT_ERR_INFO;

/*---------------------------------------------------------------------*/
/* TCACT_SLOT_INFO                                                     */
/*---------------------------------------------------------------------*/
typedef struct{                         // slot map infomation
        int         count;              // # of slot
        int         *slot_list;         // slot list
        } TCACT_SLOT_INFO;

/*---------------------------------------------------------------------*/
/* Content Map                                                         */
/*---------------------------------------------------------------------*/
typedef struct{                         // Content Map information
        int         count;              // #of lotid
        char        **lotid;            // lotid list
        char        **substid;          // substrate id list
        } TCACT_CONTENT;

/*---------------------------------------------------------------------*/
/* S3F17 : Carrier Action Information Record                           */
/*---------------------------------------------------------------------*/
typedef struct{                         // Carrier Action attr information
        char        *cattrid;           // cattrid
        int         attr_index;         // attr index
        void        *cattrdata;         // cattrdata
        } TCACT_PARA;

/*---------------------------------------------------------------------*/
typedef struct{                         // Carrier Action Infomation
        TDATAID     dataid;             // S3F17 dataid
        char        *caction;           // action name
        int         action_index;       // action index
        char        *carspec;           // carrier spec ( carid )
        int         ptn;                // port no.
        int         cp_count;           // parameter count
        TCACT_PARA  **cp_list;          // paramete list
        } TCACT_INFO;

/*===== Port Action ( S3F23, S3F25 ) ==================================*/

/*---------------------------------------------------------------------*/
/* S3F23 : Port Group Action Information Record                        */
/*---------------------------------------------------------------------*/
typedef struct{
        char        *paramname;         // paramname
        int         pval_fmt;           // paramval  item fmt
        int         pval_size;          // paramval  data array size
        void        *paramval;          // paramval data
        } TPORTG_PARA;

/*---------------------------------------------------------------------*/
typedef struct{
        char        *portgrpaction;     // group action
        char        *portgrpname;       // port group name
        int         pn_count;           // parameter count
        TPORTG_PARA **pn_list;          // paramete list
        } TPORTG_INFO;

/*---------------------------------------------------------------------*/
/* S3F25 : Port Action Information Record                              */
/*---------------------------------------------------------------------*/
typedef struct{
        char        *paramname;         // paramname
        int         pval_fmt;           // paramval  item fmt
        int         pval_size;          // paramval  data array size
        void        *paramval;          // paramval data
        } TPORT_PARA;

/*---------------------------------------------------------------------*/
typedef struct{
        char        *portaction;        // port action
        int         ptn;
        int         pn_count;           // parameter count
        TPORT_PARA  **pn_list;          // paramete list
        } TPORT_INFO;

/*===== Port Access Mode ( S3F27) =====================================*/
/*---------------------------------------------------------------------*/
typedef struct{
        int         accessmode;         // access mode 0/1
        int         port_count;         // no. of port
        int         *port_list;         // port no. list
        } TACCESS_INFO;

/*---------------------------------------------------------------------*/
/* S3F27 - Port Access Mode information                                */
/*---------------------------------------------------------------------*/
typedef struct{
        int         port;               // port no.
        int         errcode;            // ok/ng - port
        char        *errtext;           // error text - port
        } TACCESS_ERR_PORT;

/*---------------------------------------------------------------------*/
/* S3F28 ack                                                           */
/*---------------------------------------------------------------------*/
typedef struct{
        int         caack;              // ack
        int         err_count;          // # of error
        TACCESS_ERR_PORT **err_list;    // err info list
        } TACCESS_ERR_INFO;


/*===== RCP Recipe ====================================================*/
/*---------------------------------------------------------------------*/
/* Recipe Information & Parameter                                      */
/*---------------------------------------------------------------------*/
typedef struct{                         // Recipe Parameter
        char        *rcpparnm;          // para name
        int         par_fmt;            // para format
        int         par_size;           // para array size
        void        *rcpparval;         // para value;
        } TRCP_PARA;

/*---------------------------------------------------------------------*/
#define RCPST_SETUP             0       // rcp state=
#define RCPST_READY             1       // gp
#define RCPST_SELECTING         2       // gp
#define RCPST_STOP              3       // ~
/*---------------------------------------------------------------------*/

typedef struct{
        int         index;              // info index
        int         state;              // info state
        char        *name;              // name
        char        *rcpspec;           // rcpid
        int         para_count;         // # of pparameter
        TRCP_PARA   **para_list;        // parameter list
        char        *rcpbody;           // recipe body
        } TRCP_INFO;                    // Recipe Information

/*---------------------------------------------------------------------*/
/* S15Fxx Ack - Rcp Info                                           */
/*---------------------------------------------------------------------*/
typedef struct{                         // S15Fxx ack info
        int         rmack;              // U1
        int         err_count;          // # of err
        TERR_INFO   **err_list;         // error list
        } TRCP_ERR_INFO;


/*---------------------------------------------------------------------*/
/* Recipe Action S15f3 (Create / Delete )                              */
/*---------------------------------------------------------------------*/
typedef struct{                         // Recipe Action
        char        *rmnsspec;          // rcpid
        int         rmnscmd;            // action command 1=create,5=delete
        } TRCP_ACT_INFO;

/*---------------------------------------------------------------------*/
/* Recipe Rename S15f5                                                 */
/*---------------------------------------------------------------------*/
typedef struct{                         // rename info
        char        *rmnsspec;          // rcpid
        char        *rmnewns;           // new rcpid name
        } TRCP_RENAME_INFO;

/*---------------------------------------------------------------------*/
/* S15F8  Ack - Rcp Info                                           */
/*---------------------------------------------------------------------*/
typedef struct{                         // recipe name space ack
        ULONG       rmspace;            // name space
        int         rmack;              // ack (U1)
        int         err_count;          // # of error
        TERR_INFO   **err_list;         // error list
        } TRCP_S15F8_INFO;

/*---------------------------------------------------------------------*/
/* S15F9 - Recipe Status Request                                       */
/*---------------------------------------------------------------------*/
typedef struct{
        int         rcpstat;            // status U1
        char        *rcpver;            // version A
        int         rmack;              // U1
        int         err_count;          // # of error
        TERR_INFO   **err_list;         // error list
        } TRCP_S15F10_INFO;

/*---------------------------------------------------------------------*/
/* S15F18 - Recipe Retrieve Attr Data                                  */
/*---------------------------------------------------------------------*/
typedef struct{                         // Recipe Retrieve Attr Data
        char        *attrid;            // attr id
        int         format;             // attr format
        int         asize;              // attr array size
        void        *attrdata;          // attr data
        } TRCP_ATTR;

/*---------------------------------------------------------------------*/
typedef struct{                         // Recipe Retrieve Section info
        char        *rcpsecnm;          // section name
        int         attr_count;         // # of attribute data
        TRCP_ATTR   **attr_list;        // attr info list
        } TRCP_SECNM;

/*---------------------------------------------------------------------*/
typedef struct{                         // Recipe Retrieve info
        TRCP_SECNM  *m_secnm;           // section info
        char        *rcpbody;           // rcp body
        int         dset_count;         // # of section info
        TRCP_SECNM  **secnm_list;       // section info list
        int         rmack;              // rmack
        int         err_count;          // # of error
        TERR_INFO   **err_list;         // error list
        } TRCP_S15F18_INFO;

/*===== CJ, PRJ command information ===================================*/
/*---------------------------------------------------------------------*/
/* Command Parameter  S16F5, S16F27                                    */
/*---------------------------------------------------------------------*/
typedef struct{                         // Command Parmeter Information
        char        *cpname;            // cpname
        int         cpval_fmt;          // cpval item fmt
        int         cpval_size;         // cpval data array size
        void        *cpval;             // cpval
        } TCMD_PARA;

/*---------------------------------------------------------------------*/
/* Process Job State                                                   */
/*---------------------------------------------------------------------*/
#define PRST_QUEUED             0
#define PRST_SETTING_UP         1
#define PRST_WAITING_FOR_START  2
#define PRST_PROCESSING         3
#define PRST_PROCESS_COMPLETE   4
#define PRST_RESERVED           5
#define PRST_PAUSING            6
#define PRST_PAUSED             7
#define PRST_STOPPING           8
#define PRST_ABORTING           9

/*---------------------------------------------------------------------*/
/* Process Job Information (S16F11, S16F9)                             */
/*---------------------------------------------------------------------*/
typedef struct{                         // prj information
        int         index;              // info index
        int         state;              // info state
        char        *prjobid;           // prjob id
        int         mf;                 // material format code
        int         cj_index;           // control job information index
        int         car_count;          // # of carrier
        TCAR_INFO   **car_list;         // carrier id list
        int         prrecipemethod;     // fmt=51(8) U1
        TRCP_INFO   *rcp_info;          // recipe info
        int         prprocessstart;     // fmt 11(8) Bool 1=auto,0=man
        int         ceid_count;         // # of pause ceid
        TCEID       *pause_ceid_list;   // pause ceid list
        } TPRJ_INFO;

/*---------------------------------------------------------------------*/
/* S16F16 Ack - PRJob MultiCreate                                  */
/*---------------------------------------------------------------------*/
typedef struct{                         //
        int         prj_count;          // # of prj
        char        **prj_list;         // prj list
        int         acka;               // Boolean
        int         err_count;          // # of error
        TERR_INFO   **err_list;         // error list
        } TPRJ_ERR_INFO;

/*---------------------------------------------------------------------*/
/* TPRJ_LIST - S16F15                                                  */
/*---------------------------------------------------------------------*/
typedef struct{                         // Multi Prj infomation
        int         prj_count;          // # of prj
        TPRJ_INFO   **prj_list;         // prj info list
        int         err_count;          // error count
        TERR_INFO   **err_list;         // error list
        } TPRJ_LIST;

/*---------------------------------------------------------------------*/
/* S16F5 Proccess Job     Command Information                          */
/*---------------------------------------------------------------------*/
typedef struct{                         // PRJ command infomation
        char        *prjobid;           // prj id
        char        *cmd;               // command
        int         cmd_index;          // command index
        int         cp_count;           // # of parameter
        TCMD_PARA   **cp_list;          // parameter list
        } TPRJ_CMD_INFO;

/*---------------------------------------------------------------------*/
/* S16F6  Ack - Process Job Command                                */
/*---------------------------------------------------------------------*/
typedef struct{                         // Prj command ack information
        char        *prjobid;           // prjid
        int         acka;               // ack (Boolean)
        int         err_count;          // # of error
        TERR_INFO   **err_list;         // error list
        } TPRJ_CMD_ERR_INFO;

/*---------------------------------------------------------------------*/
/* S16F17 Proccess Job  Deque                                          */
/*---------------------------------------------------------------------*/
typedef struct{                         // Deque infomation
        int         prj_count;          // # of prj
        char        **prj_list;         // prjid list
        } TPRJ_DEQ_INFO;

/*---------------------------------------------------------------------*/
/* S16F18  Ack - PRJob Deque                                       */
/*---------------------------------------------------------------------*/
typedef struct{                         // Deque ack
        int         prj_count;          // # of prj dequed
        char        **prj_list;         // prjid list
        int         acka;               // ack (Boolean)
        int         err_count;          // # of error
        TERR_INFO   **err_list;         // error list
        } TPRJ_DEQ_ERR_INFO;

/*===== CJ Control Job ================================================*/

/*----- Object Type ---------------------------------------------------*/
#define EN_ControlJob           0
#define EN_Substrate            1
#define EN_xxx2                 2

/*----- Control Job ---------------------------------------------------*/
#define EN_ObjID                0
#define EN_CarrierInputSpec     1
#define EN_CurrentPRJob         2
#define EN_DataCollectionPlan   3
#define EN_MtrlOutByStatus      4
#define EN_MtrlOutSpec          5
#define EN_PauseEvent           6
#define EN_ProcessingCtrlSpec   7
#define EN_ProcessingOrderMgmt  8
#define EN_PRJobStatusList      9
#define EN_StartMethod          10
#define EN_State                11
/*---------------------------------------------------------------------*/
#define CJ_Start                1
#define CJ_Pause                2
#define CJ_Resume               3
#define CJ_Cancel               4
#define CJ_Deselect             5
#define CJ_Stop                 6
#define CJ_Abort                7
#define CJ_CJHOQ                8

/*----- Substrate Object-----------------------------------------------*/
#define SO_ObjID                0x10
#define SO_AcquiredID           0x11
#define SO_BatchLocID           0x12
#define SO_LotID                0x13
#define SO_MaterialStatus       0x14
#define SO_SubstDestination     0x15
#define SO_SubstHistory         0x16
#define SO_SubstIDStatus        0x17
#define SO_SubstLocID           0x18
#define SO_SubstProcState       0x19
#define SO_SubstSource          0x1a
#define SO_SubstState           0x1b
#define SO_SubstType            0x1c
#define SO_SubstUsage           0x1d

/*---------------------------------------------------------------------*/
#define SC_CancelSubstrate              1
#define SC_ChangeSubstrateState         2
#define SC_ProceedWithSubstrate         3
#define SC_UpdateSubstrateObject        4

/*---------------------------------------------------------------------*/
/*  MtrlOutByStatus                                                    */
/*---------------------------------------------------------------------*/
typedef struct{
        int         mtrl_status;        // material statuss (U1)
        char        *carid;             // carrier ID
        int         slot_count;         // # of slotid
        int         *slotid_list;       // slotid list
        } TMTRL_OUT_STAT;

/*---------------------------------------------------------------------*/
/*  MtrlOutSpec                                                        */
/*---------------------------------------------------------------------*/
typedef struct{
        char        *src_carid;         // source carrier id
        int         src_slot_count;     // # of slotid
        int         *src_slotid_list;   // slotid list

        char        *dst_carid;         // destination carrier id
        int         dst_slot_count;     // # of slotid
        int         *dst_slotid_list;   // slotid list
        } TMTRL_OUT_SPEC;

/*---------------------------------------------------------------------*/
/* VoidList - void ptr list                                            */
/*---------------------------------------------------------------------*/
typedef struct{
        int         count;              // # of list
        void        **void_list;        // void list
        } TVOID_LIST;

/*---------------------------------------------------------------------*/
/* Control Rule                                                        */
/*---------------------------------------------------------------------*/
typedef struct{
        char    *name;                  // rule name
        int     fmt;                    // rule value format
        int     asize;                  // array size
        void    *value;                 // rule value
        } TCTRL_RULE;

/*---------------------------------------------------------------------*/
/* Output Rule                                                         */
/*---------------------------------------------------------------------*/
typedef struct{
        int     status;                 // status (U1)
        int     fmt;                    // value format
        int     asize;                  // array size
        void    *value;                 // out rule value
        } TOUT_RULE;

/*---------------------------------------------------------------------*/
/*  CTRL_SPEC                                                          */
/*---------------------------------------------------------------------*/
typedef struct{
        char        *prjobid;           // prjid
        int         ctrl_rule_count;    // # of control rule
        TCTRL_RULE  **ctrl_rule_list;   // control rule list
        int         out_rule_count;     // # of out rule
        TOUT_RULE   **out_rule_list;    // out rule list
        } TCTRL_SPEC;

/*---------------------------------------------------------------------*/
/* Prj State                                                           */
/*---------------------------------------------------------------------*/
typedef struct{
        int     prj_count;              // # of process job
        char    **prj_list;             // prjid list
        int     *state_list;            // state list (U1)
        } TPRJ_STATE_LIST;
/*---------------------------------------------------------------------*/
/* Pause Event                                                         */
/*---------------------------------------------------------------------*/
typedef struct{
        int     ce_count;               // # of pause ceid
        int     *ceid_list;             // ceid list
        } TPAUSE_EVENT;

/*---------------------------------------------------------------------*/
/* Text infomatin  (array)                                             */
/*---------------------------------------------------------------------*/
typedef struct{
        int         text_count;         // # of text information
        char        **text_list;        // text information list
        } TCJ_TEXT_INFO;

/*---------------------------------------------------------------------*/
/* Job Control Attr Info                                               */
/*---------------------------------------------------------------------*/
typedef struct{
        char            *attrid;        // attribute id
        int             attrid_index;   // attr index
        void            *attrdata;      // attrdata
        } TOBJ_ATTR_INFO;

/*---------------------------------------------------------------------*/
/* Job Control Object Info                                             */
/*---------------------------------------------------------------------*/
typedef struct{
        int             index;          // info index
        int             state;          // info state
        int             objspec_flag;   // objspec set flag
        char            *objspec;       // objspec
        int             objtype_flag;   // obj type set flag
        char            *objtype;       // obj type
        char            *objid;         // objid (=cjid) = objspec
        int             attr_count;     // # of attribute info
        TOBJ_ATTR_INFO  **attr_list;    // attr list
        } TOBJ_INFO;

/*---------------------------------------------------------------------*/
/* S14F10 Ack - EQ --> HOST(EQ Eng)                                */
/*---------------------------------------------------------------------*/
typedef struct{
        int         objack;             // ack
        int         err_count;          // # of error
        TERR_INFO   **err_list;         // error list
        } TOBJ_ERR_INFO;

/*---------------------------------------------------------------------*/
/* S14F10 Ack EQ --> HOST(Cell Eng)                                */
/*---------------------------------------------------------------------*/
typedef struct{
        char        *objspec;           // objspec ( = objid = cjid )
        int         attr_count;         // attr count
        TOBJ_ATTR_INFO  **attr_list;    // attribute state
        int         objack;             // ack
        int         err_count;          // # of error
        TERR_INFO   **err_list;         // error list
        } TOBJ_S14_ERR_INFO;



//----- S14F19 SVCNAME index ------------------------------------------------
#define RTC_CancelMoveReticle           0
#define RTC_MoveReticle                 1
#define RTC_OktoUseReticle              2
#define RTC_RejectReticle               3
#define RTC_Re_qualifyReticle           4


//----- S14F19 Service Parameter Name index----------------------------------
#define RTCPN_ReticleID                 0
#define RTCPN_PortID                    1
#define RTCPN_DestinationLocation       2
#define RTCPN_SourceLocation            3
#define RTCPN_QualificationIntervalTime 4
#define RTCPN_Status                    5

/*---------------------------------------------------------------------*/
// S14F19: Generic Service Request Information
/*---------------------------------------------------------------------*/
typedef struct tgsv_para{
        char        *spname;            // spname
        int         spval_fmt;          // spval item fmt
        int         spval_size;         // spval data array size
        void        *spval;             // spval
        }TGSV_PARA;

/*---------------------------------------------------------------------*/
typedef struct{
        ULONG       opid;               // OPID
        char        *objspec;           // object spec
        char        *svcname;           // svcname
        int         sp_count;           // parameter count
        TGSV_PARA   **sp_list;          // parameter list
        }TS14F19_INFO;

/*---------------------------------------------------------------------*/
// S14F20  Ack - Receive
/*---------------------------------------------------------------------*/
typedef struct{
        int         svcack;             // Bin
        ULONG       linkid;             // Link ID
        int         sp_count;           // service para count
        TGSV_PARA   **sp_list;          // parameter list
        int         err_count;          // error count
        TERR_INFO   **err_list;         // error info list
        }TS14F20_ERR_INFO;

/*---------------------------------------------------------------------*/
// S14F21: Generic Service Completion Information
/*---------------------------------------------------------------------*/
typedef struct{
        ULONG       opid;               // OPID
        ULONG       linkid;             // Link ID
        int         sp_count;           // parameter count
        TGSV_PARA   **sp_list;          // paramete list
        int         svcack;             // Bin
        int         err_count;          // error count
        TERR_INFO   **err_list;         // error info list
        }TS14F21_INFO;


/*---------------------------------------------------------------------*/
/* S16F27 Control Job Command Information                              */
/*---------------------------------------------------------------------*/
typedef struct{
        char        *ctljobid;          // cjid
        int         cmd;                // 2008.10.16 *char -> int
        TCMD_PARA   *cp_info;           // command parameter
        } TCJ_CMD_INFO;

/*---------------------------------------------------------------------*/
/* S16F28 Ack                                                      */
/*---------------------------------------------------------------------*/
typedef struct{
        int         acka;               // ack
        TERR_INFO   *err_info;          // error information
        } TCJ_CMD_ERR_INFO;

/*---------------------------------------------------------------------*/
/* S14F1 - Get Attr Request                                            */
/*---------------------------------------------------------------------*/
#define OBJ_ControlJob                  0       // obj index
#define OBJ_Carrier                     1
#define OBJ_Substrate                   2

/*---------------------------------------------------------------------*/
#define ATTR_CJ_State                   0       // attr index

/*---------------------------------------------------------------------*/
#define ATTR_CarIdStatus                0
#define ATTR_CarAccessStatus            1
#define ATTR_CarSlotMapStatus           2
#define ATTR_CarlocationID              3

/*---------------------------------------------------------------------*/
#define ATTR_SubstIdStatus              0
#define ATTR_SubstLocId                 1
#define ATTR_SubstMid                   2
/*---------------------------------------------------------------------*/
typedef struct{
        int     obj_index;              // object index
        char    *objtype;               // object type
        int     attr_count;             // # of attribute
        int     *attr_index;            // attr index list
        char    **attrid;               // attrid list
        } TATTR_INFO;
/*---------------------------------------------------------------------*/
typedef struct{
        char    *id;                    // attrid
        int     index;                  // attr index
        } TOBJ_ATTR;

/*===== Terminal Message S10F5 ========================================*/
/*---------------------------------------------------------------------*/
/* S10F5 text information                                              */
/*---------------------------------------------------------------------*/
typedef struct{
        int     tid;                    // terminal id
        int     text_count;             // # of text
        char    **text_list;            // text list
        int     ackc10;                 // ack to return
        }TTERMTEXT_INFO;                // terminal text

/*===== Host Command S2F41 ============================================*/
/*---------------------------------------------------------------------*/
/* S2F41 : Remote Command Information Record                           */
/*---------------------------------------------------------------------*/
typedef struct{
        char        *cpname;            // cpname
        int         cpval_fmt;          // cpval item fmt
        int         cpval_size;         // cpval data array size
        void        *cpval;             // cpval
        }TRCMD_PARA;

/*---------------------------------------------------------------------*/
typedef struct{
        char        *rcmd;              // rcmd
        int         cp_count;           // parameter count
        TRCMD_PARA  **cp_list;          // paramete list
        }TRCMD_INFO;

/*---------------------------------------------------------------------*/
/* S2F42  Ack - S2F42 encodep                                     */
/*---------------------------------------------------------------------*/
typedef struct{
        int         hcack;              // ack (B)
        int         err_count;          // # of error
        int         *err_list;          // err position list
        int         *cpack_list;        // cp ack list
} TRCMD_ERR_INFO;

/*---------------------------------------------------------------------*/
/* S2F42  Ack - S2F42 decodep                                     */
/*---------------------------------------------------------------------*/
typedef struct{
        int         hcack;              // ack (B)
        int         err_count;          // # of error
        char        **cpname_list;      // cpname list
        int         *cpack_list;        // cp ack list
} TRCMD_HERR_INFO;

/*===== Enhanced Remote Command S2F49 =================================*/
/*---------------------------------------------------------------------*/
/* S2F49 : Enhanced Remote Command Information Record                  */
/*---------------------------------------------------------------------*/
typedef struct tercmd_para{
        char        *cpname;            // cpname
        int         cpx_count;          // > 0 cp_list =NULLȂnestingȂ
        struct tercmd_para **cpx_list;  // nesting
        int         *cpval_fmt;         // cpval item fmt
        int         *cpval_size;        // cpval data array size
        void        **cpval;            // cpval
        }TERCMD_PARA;

/*---------------------------------------------------------------------*/
typedef struct{
        char        *objspec;           // object spec
        char        *rcmd;              // rcmd
        int         cp_count;           // parameter count
        TERCMD_PARA **cp_list;          // paramete list
        }TERCMD_INFO;

/*---------------------------------------------------------------------*/
/* S2F50  Ack - for encode                                         */
/*---------------------------------------------------------------------*/
typedef struct{
        int         hcack;              // ack (B)
        int         err_count;          // # of error
        int         *err_list;          // err position list
        int         *cepack_list;       // ack list
} TERCMD_ERR_INFO;

/*---------------------------------------------------------------------*/
/* S2F50  Ack - for decode                                         */
/*---------------------------------------------------------------------*/
typedef struct{
        int         hcack;              // ack (B)
        int         err_count;          // # of error
        char        **cpname_list;      // cpname
        int         *cepack_list;       // ack list
} TERCMD_HERR_INFO;


//----- S3F35 information ---------------------------------------------

typedef struct{
        char        *attrid;
        int         fmt;
        int         size;
        BYTE        *attrdata;
} TRET_ATTR_INFO;

//---------------------------------------------------------------------
typedef struct{
        char        *ret_id;                    // reticle id
        char        *inst;                      // instrunction
} TRET_PLACE_INST_INFO;

//---------------------------------------------------------------------
typedef struct{
        char        *ret_id;                    // ret inst
        char        *inst;
        int         attr_count;
        TRET_ATTR_INFO  **attr_list;
} TRET_REMOVE_INST_INFO;

//---------------------------------------------------------------------
typedef struct{
        char        *jobaction;                 // action
        char        *podid;
        int         in_ptn;
        int         out_ptn;
        int         attr_count;
        TRET_ATTR_INFO **attr_list;
        int         remove_inst_count;
        TRET_REMOVE_INST_INFO **remove_inst_list;
        int         place_inst_count;
        TRET_PLACE_INST_INFO **place_inst_list;
} TS3F35_INFO;

//----- S3F36 information ---------------------------------------------
typedef struct{
        int         rpmack;
        int         err_count;
        TERR_INFO   **err_list;
} TS3F36_ERR_INFO;


/*===== SECS-II primary message registration and polling message =====*/
/*---------------------------------------------------------------------*/
/* TPRI_PRO_INFO : S,F and processing function ptr information         */
/*---------------------------------------------------------------------*/
typedef struct{
        int     s;                      // Stream
        int     f;                      // Function
        int     queue_flag;             // queue flag ( polling list )
        int     (*func)( int end_status, ULONG upara );
                                        // S,F msg processing function
        }TPRI_PRO_INFO;

/*---------------------------------------------------------------------*/
/* TPRI_FUNC_INFO : S,F and processing function ptr information        */
/*---------------------------------------------------------------------*/
typedef struct{
        int     sxf;                    // stream * 256 + function
        int     (*func)();       // sf processing function
        }TPRI_FUNC_INFO;

/*---------------------------------------------------------------------*/
/* Primary Msg Queue Information by Polling ==> user app               */
/*---------------------------------------------------------------------*/
typedef struct{
        ID_TR   trid;                   // DSHDR2 transaction ID
        DSHMSG  *smsg;                  //  "     primary msg info received
        DSHMSG  **prmsg;                //  "     response msg info(not used)
        int     (**callback)();  // callback funtcion
        } TMSG_QUEUE_INFO;

/*===== Backup File Information =======================================*/
/*---------------------------------------------------------------------*/
/* Backup File Information                                             */
/*  DshCkBkupXXXXInfo()֐Ŏgp  EC, SV, DVVAL, PP, CJ etc           */
/*---------------------------------------------------------------------*/

#define EI_NO_FILE  100                 // no backup file available
                                        // DshCkBkupXXXXInfo̕ԋp
typedef struct{
        char    fname[256];             // valid backup file name
        char    fname_old[256];         // original file name of valid file
        char    time_stamp[32];         // file time stamp
        int     rec_count;              // # of records
        int     file_index;             // file generation index(0,1,2 or 3 )
        } TBACKUP_INFO;

/*===== System Information - name, id list collection =================*/

#define LIST_BUFF_SIZE  0x10000

/*---------------------------------------------------------------------*/
/* System Information Collection List                                  */
/*---------------------------------------------------------------------*/
typedef struct{                         // for ID in binary
        int         count;              // # of ID
        ULONG       *id_list;           // ID list
        char        **name_list;        // name list
        }TBIN_DLIST;

/*---------------------------------------------------------------------*/
typedef struct{                         // for ID in ASCII
        int         count;              // # of ID
        char        **id_list;          // ID list
        char        **name_list;        // name list
        }TTEXT_DLIST;

/*---------------------------------------------------------------------*/
typedef struct{
        int     max_list;
        HANDLE  mutex;
        int     wtime;
        int     count;
        int     pptr;
        int     gptr;
        ULONG   **list;
}TDshList;


/*----- Enf of File --------------------------------------------------------*/
