/*---------------------------------------------------------------------------*/
/*                                                                           */
/* DSHGEMLIB GEM+GEM300ʐMGW                                          */
/*                                                                           */
/*     <               >                                                     */
/*                                                                           */
/*  Copyright (c)2003 by Data Map Corp.                                      */
/*---------------------------------------------------------------------------*/
#include    "all.h"

static  void    xAppend( ULONG p, int size, int mf );
static  void    xRemove( ULONG p );
static  void    get_mmutex( void );
static  void    rls_mmutex( void );
static  void    setup_hash_tab( void );
static  int     get_mhash( ULONG addr );
static  int     src_mhash( ULONG addr );
static  int     ghash_m( ULONG  addr );
static  int     make_hash_size( int n );
//==============================================================================
#define MALO_TIME       10
static  mutex_t  m_mutex;

static  int     mem_ck_flag = 0;
static  int     tab_size = 50000;
static  int     hash_size = 0;
static  ULONG   *hash_tab = NULL;
static  ULONG   *mark_tab = NULL;
static  int     *size_tab = NULL;
static  int     *count_tab = NULL;
static  int     count = 0;

static  FILE    *fpp = NULL;
static  FILE    *fpl = NULL;

/*----------------------------------------------*/
/* g_malloc : allocate memory                   */
/*----------------------------------------------*/
void    *g_malloc( int  size, int mf )
{
    void *p = malloc( size );
    xAppend( (ULONG)p, size, mf );
    return p;
}
/*----------------------------------------------*/
/* g_calloc : allocate memory                   */
/*----------------------------------------------*/
void    *g_calloc( int  size, int mf )
{
    void *p = calloc( size, 1 );
    xAppend( (ULONG)p, size, mf );
    return p;
}
/*----------------------------------------------*/
/* g_realloc : reallocate memory                */
/*----------------------------------------------*/
void    *g_realloc( void *ptr, int  size, int mf )
{
    void *p;
    xRemove( (ULONG)ptr );
    p = realloc( ptr, size + 100);
    xAppend( (ULONG)p, size, mf );
    return p;
}
/*----------------------------------------------*/
/* g_free : free memory                         */
/*----------------------------------------------*/
void    g_free( void *p )
{
    if ( p == NULL ){
        Gprintf( "g_free(NULL) error p=%x\n", p );
    }
    free( p );
    xRemove( (ULONG)p );
}
//**********************************************************************
/*---------------------------------------------------------------------*/
// g_mem_ck_flag() - E[N`FbNFlag
/*---------------------------------------------------------------------*/
void g_mem_ck_flag( int flag )
{
    int     ei;
    fpp = fopen("c_eqmem.log", "w" );
    fprintf( fpp, "**** c_eq mem leack check **** flag=%d\n", flag );
    fclose(fpp);
    mem_ck_flag = flag;

    if ( flag != 0 ){
        ei = mutex_init( &m_mutex, USYNC_PROCESS_ROBUST, NULL );
        setup_hash_tab();

        fpl = fopen( "c_eqmalloc.log", "w" );
        fprintf( fpl, "mutex_init() ei=%d \n", ei );
        fflush( fpl );
    }
}
/*---------------------------------------------------------------------*/
/* g_PrintMalloc()                                                     */
/*---------------------------------------------------------------------*/
void    g_PrintMalloc( void )
{
    int     x, n=0;
    if ( mem_ck_flag != 0 ){
        get_mmutex();
        fprintf( fpl, "***** memory leak information *****\n" );
        for ( x=0; x < hash_size; x++ ){
            if ( hash_tab[x] != 0 ){
                n++;
                fprintf( fpl, "%4d. 0x%x  size=%4d  mark=%d (%d)\n",
                        n, (int)hash_tab[x], size_tab[x], (int)mark_tab[x], count_tab[x] );
            }
        }
        fprintf( fpl, "    # of memory leaks = %d  (total alloc count=%d)\n", n, count );
        rls_mmutex();
    }
    if ( fpl != NULL ){
        fclose( fpl );
        fpl = NULL;
    }

    mutex_destroy( &m_mutex );

    if ( hash_tab != 0 ){
        free( hash_tab );   hash_tab = NULL;
        free( mark_tab );   mark_tab = NULL;
        free( size_tab );   size_tab = NULL;
        free( count_tab );  count_tab = NULL;
    }
}

/*---------------------------------------------------------------------*/
/* xAppend()                                                           */
/*---------------------------------------------------------------------*/
static  void    xAppend( ULONG p, int size, int mf )
{
    int     x;
    if ( mem_ck_flag == 0 ) return;
    get_mmutex();
    x = get_mhash( p );
    if ( x >= 0 ){
        mark_tab[x] = mf;
        count_tab[x] = count++;
        size_tab[x] = size;
    }
    else{
        fprintf( fpl, "!!  Error p=0x%x mark=%d hash tab overflow count=%d\n", (int)p, mf, count );
        fflush( fpl );
    }
    rls_mmutex();
}
/*---------------------------------------------------------------------*/
/* xRemove()                                                           */
/*---------------------------------------------------------------------*/
static  void    xRemove( ULONG p )
{
    int     x;
    if ( mem_ck_flag == 0 ) return;
    get_mmutex();
    x = src_mhash( (ULONG)p );
    if ( x < 0 ){
        fprintf( fpl, "!!  Error p=0x%x not allocated count=%d\n", (int)p, count );
        fflush( fpl );
    }
    else{
        hash_tab[x] = 0;
        mark_tab[x] = 0;
    }
    rls_mmutex();
}
/*---------------------------------------------------------------------*/
/* get_mmutex()                                                         */
/*---------------------------------------------------------------------*/
static  void    get_mmutex( void )
{
        mutex_lock( &m_mutex );
}
/*---------------------------------------------------------------------*/
/* rls_mmutex()                                                         */
/*---------------------------------------------------------------------*/
static  void    rls_mmutex( void )
{
        mutex_unlock( &m_mutex );
}
/*---------------------------------------------------------------------------*/
/* setup_hash_tab()                                                          */
/*---------------------------------------------------------------------------*/
static  void    setup_hash_tab( void )
{
    if ( hash_tab != NULL ) free( hash_tab );
    if ( mark_tab != NULL ) free( mark_tab );

    hash_size = make_hash_size( tab_size );
    hash_tab  = calloc( sizeof(ULONG), hash_size );
    mark_tab  = calloc( sizeof(ULONG), hash_size );
    size_tab  = calloc( sizeof(ULONG), hash_size );
    count_tab = calloc( sizeof(ULONG), hash_size );
}

/*---------------------------------------------------------------------------*/
/* get_mhash : get and put binary hash                                       */
/*---------------------------------------------------------------------------*/
static int get_mhash( ULONG addr )
{
        int     x, hashed;
    x = hashed = ghash_m( addr );
    while ( hash_tab[x] !=0 ){
        x = ( x + ( hash_size / 2 - 1 ) ) & ( hash_size - 1 );
        if ( hashed == x ){ return (-1); }
    }
    hash_tab[x] = addr;
    return x;
}
/*---------------------------------------------------------------------------*/
/* src_mhash : search  hash                                                  */
/*---------------------------------------------------------------------------*/
static int src_mhash( ULONG addr )
{
        int     x, hashed;
    x = hashed = ghash_m( addr );
    do{
        if ( hash_tab[x] != 0 ){ if ( hash_tab[x] == addr ) return x; }
        x = ( x + ( hash_size / 2 - 1 ) ) & ( hash_size - 1 );
    } while(  hashed != x );
    return (-1);
}
/*---------------------------------------------------------------------------*/
/* ghash_m : generate hash value                                             */
/*---------------------------------------------------------------------------*/
static  int  ghash_m( ULONG  addr )
{
    return ( (addr >> 2 ) & ( hash_size - 1 ) );
}
/*---------------------------------------------------------------------------*/
/* make_hash_size : Hash\̃TCY߂                                   */
/*---------------------------------------------------------------------------*/
static int make_hash_size( int n )
{
        int     size;
    for ( size = 4; ; ){
        if ( n <= size ) return size;
        size = size * 2;
    }
}
/*------ End Of File --------------------------------------------------------*/
