/*--------------------------------------------------------------------------*/
/*                                                                          */
/*  DshEngLib - GEM, GEM300 Library Program Package                         */
/*                                                                          */
/*    d_r3f17.c  - wafer processing simulation                              */
/*                                                                          */
/*  Copyright (c)2007 by Data Map Corp.                                     */
/*                                                                          */
/*--------------------------------------------------------------------------*/
#include    "all.h"

/*-----------------------------------------------------------*/
// sim_s3f17_proc()
/*-----------------------------------------------------------*/

int     sim_s3f17_proc( TMSG_QUEUE_INFO *qmsg, char *carid, int option )
{
        int     ei;
        TCACT_INFO tinfo;
        int     ack = 0;
    ei = DshDecodeS3F17( qmsg->smsg, &tinfo );
    if ( ei == 0 ){                             // decode = OK
        if ( strcmp( tinfo.carspec, carid ) == 0 ){
            if ( tinfo.action_index == CA_ProceedWithCarrier ){
                ;
            }else{
                ack = 1;
                ei  = (-1);
            }
        }else{
            ack = 1;
        }
        DshFreeTCACT_INFO( &tinfo );
    }else{
        ack = 1;
    }
    s3f17_response( qmsg, ack );          // send response - see g_pmsgpr.c
    return ei;
}
/*-----------------------------------------------------------*/
// sim_s3f17_WithCarrier()
/*-----------------------------------------------------------*/
void    sim_S3F17_ProceedWithCarrier( TCACT_INFO *info, int option )
{
        int     ei = 0;
        int     slot_count;
        int     i;
        char    id[81];
        char    *carid;
        char    substid [81];
//        TTSIM_OPERATION *sim_p;
    carid = info->carspec;
    ei = sim_DoS3F17ParaProc( info, option );
    slot_count = 0;
    ei = EngGetCarSlotCount( carid, &slot_count );
    for ( i = 0; i < slot_count; i++ ){                 // set substid for each slot
        ei = EngGetCarSubstid( carid, i, substid );
        if ( ei == 0 && strlen( substid ) > 0 ){
            int ope_index = 0;
            ei = EngAllocSubstInfo( substid, &ope_index );
            if ( ei == 0 ){
                char lotid[80];
                ei = EngGetCarLotid( carid, i, lotid );
                if ( ei == 0 ){
                    ei = EngSetSubstLotIdX( ope_index, lotid );
                }
            }
        }
    }
    ei = EngGetCarLotid( carid, 0, id );
    ei = EngSetSvVal( SV_LotId, id, strlen( id ) );   // lotid
    ei = EngGetCarSubstid( carid, 0, id );
    ei = EngSetSvVal( SV_SubstID, id, strlen( id ) ); // substid
    ei = EngGetCarUsage( carid, id );
    ei = EngSetSvVal( SV_CarUsage, id, strlen( id ) );// usage
    ei = EngGetCarLocation( carid, id );
    ei = EngSetSvVal( SV_CarLocation, id, strlen( id ) ); // location

}
/*-----------------------------------------------------------*/
// sim_DoS3F17ParaProc()
/*-----------------------------------------------------------*/
int     sim_DoS3F17ParaProc( TCACT_INFO *info, int option )
{
        int     ei = 0;
        int     i;
        TCACT_PARA  *tinfo;
    if ( info->cp_count > 0 ){
        for ( i=0; ei == 0 && i < info->cp_count; i++ ){
            tinfo = info->cp_list[i];
            switch ( info->action_index ){
            case CA_Bind:
            case CA_CancelBind:
//            case CA_CancelAllCarrierOut:
            case CA_CancelCarrier:
            case CA_CancelCarrierAtPort:
            case CA_CancelCarrierNotification:
            case CA_CancelCarrierOut:
            case CA_CarrierIn:
            case CA_CarrierNotification:
            case CA_CarrierOut:
            case CA_CarrierReCreate:
            case CA_CarrierRelease:
                break;
            case CA_ProceedWithCarrier:
                ei = sim_DoParaProceedWithCarrier( info, tinfo, option );
                break;
            default:
                break;
            }
        }
    }
    return ei;
}
/*-----------------------------------------------------------*/
// sim_DoParaProceedWithCarrier()
/*-----------------------------------------------------------*/
int     sim_DoParaProceedWithCarrier( TCACT_INFO *info, TCACT_PARA *pinfo, int option )
{
        int     ei = 0;
        int     *iptr;
        int     i;
        TCACT_CONTENT   *content;
        TCACT_SLOT_INFO *sinfo;
//        TTSIM_OPERATION *sim_p;
        char    *carid = info->carspec;
//    sim_p = get_sim_operation_info( );
//    carid = sim_p->carid;
    switch ( pinfo->attr_index ){
    case CA_ObjId:
    case CA_Capacity:
          break;
    case CA_CarrierAccessingStatus:
          ei = EngSetCarAccessStatus( carid, ( int )( pinfo->cattrdata ) );
          break;
    case CA_CarrierIDStatus:
          ei = EngSetCarIdStatus( carid, ( int )( pinfo->cattrdata ) );
          break;
    case CA_ContentMap:
          content = ( TCACT_CONTENT* )pinfo->cattrdata;
          ei = EngSetCarContent( carid, content );
          break;
    case CA_LocationID:
         if ( pinfo->cattrdata == NULL ) break;
         ei = EngSetCarLocation( carid, ( char* )pinfo->cattrdata );
         break;
    case CA_SlotMap:
         sinfo = ( TCACT_SLOT_INFO* )pinfo->cattrdata;
         if ( sinfo->count > 0 ){
            iptr = sinfo->slot_list;
            for( i = 0; i < sinfo->count; i++, iptr++ ){
                ei = EngSetCarSlotmap( carid, i, *iptr );
                if ( ei < 0 ){
                    break;
                }
            }
         }
         break;
    case CA_SlotMapStatus:
        ei = EngSetCarMapStatus( carid, ( int )pinfo->cattrdata );
        break;
    case CA_SubStrateCount:
        break;
    case CA_Usage:
         if ( pinfo->cattrdata == NULL ) break;
        ei = EngSetCarUsage( carid, ( char* )pinfo->cattrdata );
        break;
    default:
        break;
    }
    return ei;
}
