/****************************************************************/
/*                                                              */
/*    << DSHENG4 Test Program >>                                */
/*                                                              */
/*              file name : g_schedu.c                          */
/*                                                              */
/*  Produced by Data Map Corporation in 2009.                   */
/*  All rights reserved. No part of this program may be copied  */
/*  in anyway, or by any means, without permission from         */
/*  Data Map Corporation.                                       */
/*                                                              */
/****************************************************************/
#include    "all.h"

/*-----------------------------------------------------------*/
#define BUF_SIZE    10000

int     sim_job_count;
TTSIM_JOB_LIST  *sim_job_list;        // Job Schedule List

/*-----------------------------------------------------------*/
// setup_job_list()
/*-----------------------------------------------------------*/
int     setup_job_list( char *fname, char *errstr )
{
        int     n;
        char    *p;
    sim_job_count = n = get_no_of_job( fname, errstr );
    if ( n == 0 ){
        Gprintf( " no job,  errstr = %s\n", errstr );
        return (-1);
    }
    alloc_job_list( n );
    p = g_malloc( BUF_SIZE, 8500 );
    n = make_job_list( fname, errstr );
    if ( n < 0 ){
        Gprintf( "job command error, errstr = %s\n", errstr );
        free_job_list( );
    }
    return n;
}
/*-----------------------------------------------------------*/
// alloc_job_list()
/*-----------------------------------------------------------*/
void    alloc_job_list( int n )
{
        TTSIM_JOB_LIST  *s_list;
    s_list = sim_job_list = g_calloc( sizeof(TTSIM_JOB_LIST), 8010 );
    s_list->job_count = n;
    s_list->job_index = 0;
    s_list->job_list = g_calloc( sizeof(void*)*n, 8011 );
}
/*-----------------------------------------------------------*/
// alloc_job_info()
/*-----------------------------------------------------------*/
TTJOB_INFO *alloc_job_info( void )
{
        int     i;
        TTSIM_JOB_LIST  *s_list = sim_job_list;
    for ( i=0; i < s_list->job_count; i++ ){
        if ( s_list->job_list[i] != NULL ) continue;
        s_list->job_list[i] = g_calloc( sizeof(TTJOB_INFO), 8002 );
        s_list->job_list[i]->para_count = K_MAX_PARA;
        return s_list->job_list[i];
    }
    return NULL;
}

/*-----------------------------------------------------------*/
// free_job_list()
/*-----------------------------------------------------------*/
void    free_job_list( void )
{
        int     i;
        TTSIM_JOB_LIST  *s_list;
    s_list = sim_job_list;

   if ( s_list==NULL ) return;
   if ( s_list->job_count > 0 ){
        for ( i=0; i < s_list->job_count; i++ ){
            free_job_info( s_list->job_list[i] );
            s_list->job_list[i] = NULL;
        }
        s_list->job_count = 0;
    }
    g_free( s_list );
    sim_job_list = NULL;
}
/*-----------------------------------------------------------*/
// free_job_info()
/*-----------------------------------------------------------*/
void    free_job_info( TTJOB_INFO *info )
{
        int     i;
        TTSLOT_INFO *sinfo;
        TTRCP_PARA  *rinfo;

    if ( info == NULL ) return;
    if ( info->carid != NULL ){ g_free( info->carid ); }
    if ( info->usage != NULL ){ g_free( info->usage ); }
    if ( info->slot_info != NULL ){
        for ( i=0; i < K_MAX_SLOT; i++ ){
            if ( (sinfo=info->slot_info[i]) == NULL ) continue;
            if ( sinfo->mid != NULL ) g_free( sinfo->mid );
            if ( sinfo->substid != NULL ) g_free( sinfo->substid );
            if ( sinfo->location != NULL ) g_free( sinfo->location );
            g_free( sinfo );
        }
    }
    if ( info->cjid != NULL ){ g_free( info->cjid ); }
    if ( info->prjid != NULL ){ g_free( info->prjid ); }
    if ( info->rcpid != NULL ){ g_free( info->rcpid ); }
    if ( info->rcpbody != NULL ){ g_free( info->rcpbody ); }
    if ( info->rcp_para != NULL ){
        for ( i=0; i < info->para_count; i++ ){
            if ( (rinfo = info->rcp_para[i]) == NULL ) continue;
            if ( rinfo->name != NULL ){ g_free( rinfo->name ); }
            if ( rinfo->value != NULL ){ g_free( rinfo->value ); }
            g_free( rinfo );
        }
        g_free( info->rcp_para );
    }
    g_free( info );
}
/*-----------------------------------------------------------*/
// put_slot_info()
/*-----------------------------------------------------------*/
int     put_slot_info( TTJOB_INFO *info, int slotpos, int slotid,
                        char *mid, char *substid, char *location )
{
        TTSLOT_INFO *sinfo;
    if ( slotpos < 0 || slotpos >= K_MAX_SLOT ) return (-1);
    sinfo = info->slot_info[slotpos] = g_calloc( sizeof(TTJOB_INFO), 8004 );
    sinfo->slotid = slotid;

    if ( mid != NULL ){
        if ( strlen(mid) > 0 ){
            sinfo->mid = g_malloc( strlen(mid)+1, 8501 );
            strcpy( sinfo->mid, mid );
        }
    }
    if ( substid != NULL ){
        if ( strlen(substid) > 0 ){
            sinfo->substid = g_malloc( strlen(substid)+1, 8502 );
            strcpy( sinfo->substid, substid );
        }
    }
    if ( location != NULL ){
        if ( strlen(location) > 0 ){
            sinfo->location = g_malloc( strlen(location)+1, 8503 );
            strcpy( sinfo->location, location );
        }
    }
    return 0;
}

/*-----------------------------------------------------------*/
// put_rcp_para()
/*-----------------------------------------------------------*/
int     put_rcp_para( TTJOB_INFO *info, char *name, char *value )
{
        TTRCP_PARA *rinfo;
        int     x;
    if ( ( x=info->para_index) >= info->para_count ) return (-1);
    rinfo = info->rcp_para[x] = g_calloc( sizeof(TTRCP_PARA), 8006 );
    if ( name != NULL ){
        if ( strlen(name) > 0 ){
            rinfo->name = g_malloc( strlen(name)+1, 8504 );
            strcpy( rinfo->name, name );
        }
    }
    if ( value != NULL ){
        if ( strlen(value) > 0 ){
            rinfo->value = g_malloc( strlen(value)+1, 8505 );
            strcpy( rinfo->value, value );
        }
    }
    info->para_index ++;
    return 0;
}


/*-----------------------------------------------------------*/
// get_no_of_job()
/*-----------------------------------------------------------*/
int     get_no_of_job( char *fname, char *errstr )
{
        int     n = 0;
        int     f;
        FILE    *fp;
        char    *buff;
    strcpy( errstr, "" );
    fp = fopen( fname, "r" );
    if ( fp == NULL ){ strcpy( errstr, "file not found" ); return 0; }
    buff = g_malloc( BUF_SIZE, 8510 );
    while ( 1 ){
        if ( fgets( buff, BUF_SIZE, fp ) ==NULL ) break;
        s_trim( buff );
        s_to_upper( buff );
        if ( strcmp( buff, "$START" )==0 ){
            f = 0;
            while( 1 ){
                if ( fgets( buff, BUF_SIZE, fp ) ==NULL ) break;
                s_trim( buff );
                s_to_upper( buff );
//Gprintf( "buff-----=%s\n", buff );
                if ( strcmp( buff, "$END" )== 0 ){
                    f = 1;
                    break;
                }
            }
            if ( f==0 ) break;
            n++;
        }
    }
    fclose( fp );
    g_free( buff );
    Gprintf( "        # of job = %d\n", n );
    return n;
}

/*-----------------------------------------------------------*/
// make_job_list()
/*-----------------------------------------------------------*/
int     make_job_list( char *fname, char *errstr )
{
        int     n = 0, ei = 0;
        FILE    *fp;
        char    *buff;
//Gprintf( "make_job_list() fname=%s\n", fname );
    errstr[0] = 0;
    fp = fopen( fname, "r" );
    if ( fp == NULL ) return 0;
    buff = g_malloc( BUF_SIZE, 8511 );
    while ( 1 ){
        if ( fgets( buff, BUF_SIZE, fp ) ==NULL ) break;
//Gprintf( " len=%d buff=%s\n", strlen(buff), buff );
        s_trim( buff );
        s_to_upper( buff );

        if ( strcmp( buff, "$START" )==0 ){
            ei = make_a_job( fp, errstr );
            if ( ei < 0 ) break;
        }
    }
//Gprintf( "make_job_list end \n" );
    fclose( fp );
    g_free( buff );
    if ( ei < 0 ){
        free_job_list( );
    }
    return ei;
}


/*-----------------------------------------------------------*/
char    *job_cmd_tab[] = {
    "carrier",
    "control_job",
    "process_job",
    "recipe",
    "id",
    "port",
    "location",
    "usage",
    "slot",
    "body",                         // for pp & rcp     s7f3, s15f13
    "f_ccode",                      // for fpp          s7f23
    "para",                         // for rcp          s15f13
    "carid",
    "ppid",                         // for pp, fpp & rcp 17f3, s7f23, f15f13
    "prjid",
    "cjid",
    "}",
    NULL
    };

/*-----------------------------------------------------------*/
char    delim_tab_1[] = { ' ', '\t', '{',  0 };
char    delim_tab_2[] = { ' ', '\t', ':', ',',  0 };
char    **token_list;

enum JOB_CMD{
    CMD_carrier,
    CMD_control_job,
    CMD_process_job,
    CMD_recipe,
    SUBCMD_id,
    SUBCMD_port,
    SUBCMD_location,
    SUBCMD_usage,
    SUBCMD_slot,
    SUBCMD_body,
    SUBCMD_f_ccode,
    SUBCMD_para,
    SUBCMD_carid,
    SUBCMD_ppid,
    SUBCND_prjid,
    SUBCMD_cjid,
    SUBCMD_end
};
/*-----------------------------------------------------------*/
// make_a_job()
/*-----------------------------------------------------------*/
int     make_a_job( FILE *fp, char *errstr )
{
        int     n;
        int     ei;
        char    *buff;
        enum JOB_CMD cmd;
        TTJOB_INFO  *info;
        TTSIM_JOB_LIST  *s_list;
    info = alloc_job_info( );
    if ( info == NULL ){
        sprintf( errstr, "no more job info area\n" );
        return (-1);
    }
    s_list = sim_job_list;
    buff = g_malloc( BUF_SIZE, 8502 );
    token_list = g_calloc( sizeof(void*)*128, 8007 );
    while( 1 ){
        ei = 0;
        if ( fgets( buff, BUF_SIZE, fp ) == NULL ){ ei = (-1); break; }
        s_trim( buff );
        if ( memcmp( buff, "$END", 4 ) == 0 ) break;
        if ( memcmp( buff, "//", 2 ) == 0 ) continue;

        n = develop_token( buff, delim_tab_1, token_list, 128 );
        if ( n==0 ) continue;

        cmd = get_job_cmd( token_list[0], job_cmd_tab );
        if ( cmd == (-1) ){
            sprintf( errstr, "cmd err-1, %s", token_list[0] );
            ei = (-1);
            break;
        }
        switch( cmd ){
        case CMD_carrier:
            ei = proc_carrier( info, fp, buff, token_list, 128, errstr );
            break;
        case CMD_control_job:
            ei = proc_control_job( info, fp, buff, token_list, 128, errstr );
            break;
        case CMD_process_job:
            ei = proc_process_job( info, fp, buff, token_list, 128, errstr );
            break;
        case CMD_recipe:
            ei = proc_recipe( info, fp, buff, token_list, 128, errstr );
            break;
        default:
            ei = (-1);
            sprintf( errstr, " main cmd err, %s\n", token_list[0] );
            break;
        }
        free_token_list( token_list, n );
        if ( ei < 0 ) break;
    }
    free_token_list( token_list, n );
    g_free( buff );
    g_free( token_list );
    if ( ei == 0 ){
        if ( info->carid == NULL ){
            sprintf( errstr, " no carrier id\n" );
            ei = (-1);
        }else if ( info->cjid == NULL ){
            sprintf( errstr, " no control job id\n" );
            ei = (-1);
        }else if ( info->prjid == NULL ){
            sprintf( errstr, " no process job id\n" );
            ei = (-1);
        }else if ( info->rcpid == NULL ){
            sprintf( errstr, " no recipe id\n" );
            ei = (-1);
        }else{
            s_list->job_list[s_list->job_index] = info;
            s_list->job_index++;
        }
    }
    if ( ei < 0 ){
        Gprintf( "!!! job schedule file error-  %s\n", errstr );
    }else{
        ; // ;;s_list->job_list[s_list->job_index++] = info;
    }
    return ei;
}

/*-----------------------------------------------------------*/
// develop_token()
/*-----------------------------------------------------------*/
int     develop_token( char *buff, char *ctl_list, char *token[], int max_token )
{
        int     i, j, n, len;
        char    *cbuf;
    cbuf = g_malloc( BUF_SIZE, 8503 );
    s_trim( buff );
    len = strlen( buff );
    for ( n=i=0; i < len && n < max_token; i++ ){
        cbuf[0] = 0;
        for ( j=0; i < len; ){
            if ( buff[i] == '\"' ){
                i++;
                while( i < len && buff[i] != '\"' ){
                    cbuf[j++] = buff[i++];
                    cbuf[j] = 0;
                }
                break;
            }
            if ( check_delimiter( buff[i], ctl_list ) == 0 ){
                break;
            }else{
                cbuf[j++] = buff[i++];
                cbuf[j] = 0;
            }
        }
        if ( strlen(cbuf) > 0 ){
            token[n] = g_malloc( strlen(cbuf)+1, 8505 );
            strcpy( token[n++], cbuf );
        }
    }
    g_free( cbuf );
    return n;
}
/*-----------------------------------------------------------*/
// check_delimiter()
/*-----------------------------------------------------------*/
int     check_delimiter( char c, char *delim_list )
{
    while( *delim_list != 0 ){
        if ( c == *delim_list ) return 0;
        delim_list++;
    }
    return (-1);
}

/*-----------------------------------------------------------*/
// get_job_cmd()
/*-----------------------------------------------------------*/
int     get_job_cmd( char *buff, char *cmd_list[] )
{
        int     i;
    s_to_lower( buff );
    for ( i=0; cmd_list[i] != NULL; i++ ){
        s_to_lower( buff );
        if ( strcmp( buff, cmd_list[i] ) == 0 ) return i;
    }
    return (-1);
}
/*-----------------------------------------------------------*/
// free_token_list()
/*-----------------------------------------------------------*/
void    free_token_list( char *list[], int n )
{
        int     i;
    for ( i=0; i < n; i++ ){
        if ( list[i] != NULL ){
            g_free( list[i] );
            list[i] = NULL;
        }
    }
}
/*-----------------------------------------------------------*/
// proc_carrier()
/*-----------------------------------------------------------*/
int     proc_carrier( TTJOB_INFO *info, FILE *fp, char *buff,
                char *token_list[], int max_list, char *errstr )
{
        int     n;
        int     i;
        int     end_flag = 0;
        int     ei = 0;
        int     cmd;
        char    mid[128];
        char    substid[128];
        char    location[128];
        int     slotid;
        int     slotno;
    while ( end_flag==0 && ei==0 ){
        free_token_list( token_list, max_list );
        if ( fgets( buff, BUF_SIZE, fp )==NULL ){
            sprintf( errstr, " carrier : not ended normally\n" );
            return (-1);
        }
        s_trim( buff );

        if ( strlen(buff)==0 ) continue;
        if ( memcmp( buff, "//", 2 ) == 0 ) continue;
        n = develop_token( buff, delim_tab_2, token_list, max_list );
        cmd = get_job_cmd( token_list[0], job_cmd_tab );
        if ( cmd < SUBCMD_end && n < 2 ){
            sprintf( errstr, "carrier : no. of parameter err\n" );
            return (-1);
        }
        if ( cmd == (-1) ){
            sprintf( errstr, "carrier :  err, %s", token_list[0] );
            return (-1);
        }
        switch( cmd ){
        case SUBCMD_id:
            if ( strlen(token_list[1]) < 1 ){
                sprintf( errstr, "carid length error\n" );
                ei = (-1);
            }else{
                info->carid = g_malloc( strlen(token_list[1])+1, 8506 );
                strcpy( info->carid, token_list[1] );
            }
            break;
        case SUBCMD_port:
            info->port = atoi( token_list[1] );
            if ( info->port < 1 || info->port > 2 ){
                sprintf( errstr, "carrier : port number err, %d\n", info->port );
                ei = (-1);
            }
            break;
        case SUBCMD_usage:
            info->usage = g_malloc( strlen(token_list[1])+1, 8507 );
            strcpy( info->usage, token_list[1] );
            break;
        case SUBCMD_slot:
            if ( n < 3 ){
                sprintf( errstr, " carrier slot, short token\n" );
                ei = (-1);
                break;
            }
            mid[0] = substid[0] = location[0] = 0;
            for ( i=1; i < n && ei==0; i++ ){
                switch( i ){
                case 1:
                    slotno = atoi( token_list[i] );
                    if ( slotno < 0 || slotno > K_MAX_SLOT ){
                        ei = (-1);
                    }
                    break;
                case 2:
                    slotid = atoi( token_list[i] );
                    break;
                case 3:
                    strcpy( mid, token_list[i] );
                    break;
                case 4:
                    strcpy( substid, token_list[i] );
                    break;
                case 5:
                    strcpy( location, token_list[i] );
                    break;
                default:
                    break;
                }
            }
            if ( ei == 0 ){
                ei = put_slot_info( info, slotno, slotid, mid, substid, location );
            }
            break;
        case SUBCMD_end:
            end_flag = 1;
            break;
        default:
            sprintf( errstr, "carrier command error\n" );
            ei = (-1);
            end_flag = 1;
            break;
        }
        free_token_list( token_list, max_list );
    }
    return ei;
}
/*-----------------------------------------------------------*/
// proc_control_job()
/*-----------------------------------------------------------*/
int     proc_control_job( TTJOB_INFO *info, FILE *fp, char *buff,
                char *token_list[], int max_list, char *errstr )
{
        int     n;
        int     end_flag = 0;
        int     cmd;
        int     ei = 0;
    while ( end_flag==0 && ei==0 ){
        free_token_list( token_list, max_list );
        if ( fgets( buff, BUF_SIZE, fp )==NULL ){
            sprintf( errstr, " control_job : not ended normally\n" );
            return (-1);
        }
        s_trim( buff );
        if ( strlen(buff)==0 ) continue;
        if ( memcmp( buff, "//", 2 ) == 0 ) continue;
        n = develop_token( buff, delim_tab_2, token_list, max_list );

        cmd = get_job_cmd( token_list[0], job_cmd_tab );
        if ( cmd != SUBCMD_end && n < 2 ){
            sprintf( errstr, "control_job : no. of parameter err\n" );
            return (-1);
        }
        switch( cmd ){
        case SUBCMD_id:
            if ( strlen(token_list[1]) < 1 ){
                sprintf( errstr, "control job length error\n" );
                ei = (-1);
            }else{
                info->cjid = g_malloc( strlen(token_list[1])+1, 8508 );
                strcpy( info->cjid, token_list[1] );
            }
            break;
        case SUBCMD_end:
            end_flag = 1;
            break;
        default:
            ei = (-1);
            sprintf( errstr, "control job command error\n" );
            end_flag = 1;
            break;
        }
        free_token_list( token_list, max_list );
    }
    return ei;
}
/*-----------------------------------------------------------*/
// proc_process_job()
/*-----------------------------------------------------------*/
int     proc_process_job( TTJOB_INFO *info, FILE *fp, char *buff,
                char *token_list[], int max_list, char *errstr )
{
        int     n;
        int     end_flag = 0;
        int     cmd;
        int     ei = 0;
    while ( end_flag==0 && ei==0 ){
        free_token_list( token_list, max_list );
        if ( fgets( buff, BUF_SIZE, fp )==NULL ){
            sprintf( errstr, " process_job : not ended normally\n" );
            return (-1);
        }
        s_trim( buff );
        if ( strlen(buff)==0 ) continue;
        if ( memcmp( buff, "//", 2 ) == 0 ) continue;
        n = develop_token( buff, delim_tab_2, token_list, max_list );

        cmd = get_job_cmd( token_list[0], job_cmd_tab );
        if ( cmd < SUBCMD_end && n < 2 ){
            sprintf( errstr, "process_job : no. of parameter err\n" );
            return (-1);
        }
        switch( cmd ){
        case SUBCMD_id:
            if ( strlen(token_list[1]) < 1 ){
                sprintf( errstr, "control job length error\n" );
                ei = (-1);
            }else{
                info->prjid = g_malloc( strlen(token_list[1])+1, 8509 );
                strcpy( info->prjid, token_list[1] );
            }
            break;
        case SUBCMD_end:
            end_flag = 1;
            break;
        default:
            sprintf( errstr, "process job command error\n" );
            ei = (-1);
            end_flag = 1;
            break;
        }
        free_token_list( token_list, max_list );
    }
    return ei;
}

/*-----------------------------------------------------------*/
// proc_recipe()
/*-----------------------------------------------------------*/
int     proc_recipe( TTJOB_INFO *info, FILE *fp, char *buff,
                char *token_list[], int max_list, char *errstr )
{
        int     n;
        int     i, j;
        int     end_flag = 0;
        int     ei = 0;
        int     cmd;
        char    name[128];
        char    value[128];
        int     c_count = 0;
        TTFPP_CCODE *ccinfo;
    while ( end_flag==0 && ei==0 ){
        free_token_list( token_list, max_list );
        if ( fgets( buff, BUF_SIZE, fp )==NULL ){
            sprintf( errstr, " recipe : not ended normally\n" );
            return (-1);
        }
        s_trim( buff );
        if ( strlen(buff)==0 ) continue;
        if ( memcmp( buff, "//", 2 ) == 0 ) continue;
        n = develop_token( buff, delim_tab_2, token_list, max_list );
        cmd = get_job_cmd( token_list[0], job_cmd_tab );
        if ( cmd < SUBCMD_end && n < 2 ){
            sprintf( errstr, "recipe : no. of parameter err\n" );
            return (-1);
        }
        switch( cmd ){
        case SUBCMD_id:
            if ( strlen(token_list[1]) < 1 ){
                sprintf( errstr, "rcpid length error\n" );
                ei = (-1);
            }else{
                info->rcpid = g_malloc( strlen(token_list[1])+1, 8510 );
                strcpy( info->rcpid, token_list[1] );
            }
            break;
        case SUBCMD_body:
            info->rcpbody = g_malloc( strlen(token_list[1])+1, 8511 );
            strcpy( info->rcpbody, token_list[1] );
            break;
        case SUBCMD_para:
            if ( n < 3 ){
                sprintf( errstr, " recipe para, short token\n" );
                ei = (-1);
                break;
            }
            name[0] = value[0] = 0;
            for ( i=1; i < 3 && ei==0; i++ ){
                switch( i ){
                case 1:
                    strcpy( name, token_list[i] );
                    break;
                case 2:
                    strcpy( value, token_list[i] );
                    break;
                default:
                    break;
                }
            }
            if ( ei == 0 ){
                ei = put_rcp_para( info, name, value );
                if ( ei < 0 ){
                    sprintf( errstr, "!!! recipe parameter error\n" );
                }
            }
            break;
        case SUBCMD_f_ccode:
            if ( info->ccode_count >= K_MAX_PARA ){
                sprintf( errstr, "!!! fpp command code over flow\n" );
                ei = (-1);
                break;
            }
            if ( n < 2 ){
                sprintf( errstr, "!!! fpp command code error\n" );
                ei = (-1);
                break;
            }
            ccinfo = g_calloc( sizeof(TTFPP_CCODE), 8010 );
            ccinfo->ccode = g_calloc( strlen(token_list[1])+1, 8011 );


            strcpy( ccinfo->ccode, token_list[1] );             // ccode
            ccinfo->para_count = n-2;                           // c肪parameter
            for ( i=2, j=0; i < n; i++, j++ ){
                strcpy( name, token_list[i] );
                ccinfo->para[j] = g_calloc( strlen(token_list[i])+1, 8012 );
                strcpy( ccinfo->para[j], token_list[i] );
            }
            info->fpp_ccode[info->ccode_count++] = ccinfo;
            break;
        case SUBCMD_end:
            end_flag = 1;
            break;
        default:
            sprintf( errstr, "recipe command error\n" );
            ei = (-1);
            end_flag = 1;
            break;
        }
        free_token_list( token_list, max_list );
    }
    return ei;
}

/*-----------------------------------------------------------*/
// get_sim_job_list_ptr()
/*-----------------------------------------------------------*/
TTSIM_JOB_LIST *get_sim_list_ptr( void )
{
    return sim_job_list;

}

/*-----------------------------------------------------------*/
// s_to_upper()
/*-----------------------------------------------------------*/
char    *s_to_upper( char *str )
{
        char    *p;
    p = str;
    while( (*str) != 0 ){
        if ( (*str) >= 'a' && (*str) <= 'z' ) (*str) &= 0xdf;
        str++;
    }
    return p;
}

/*-----------------------------------------------------------*/
// s_to_lower()
/*-----------------------------------------------------------*/
char    *s_to_lower( char *str )
{
        char    *p;
    p = str;
    while( (*str) != 0 ){
        if ( (*str) >= 'A' && (*str) <= 'Z' ) (*str) |= 0x20;
        str++;
    }
    return p;
}

/*-----------------------------------------------------------*/
// s_trim()
/*-----------------------------------------------------------*/
char    *s_trim( char *str )
{
        char    *p;
        int     n, i;
    n = strlen(str);
    p = g_malloc( n+10, 8520 );
    for ( i=0; i < (int)strlen(str); i++ ){
        if ( str[i] ==' ' || str[i]=='\t' || str[i]=='\n' || str[i]=='\r' ) continue;
        else break;
    }
    strcpy( p, &str[i] );
    n = strlen( p );
    if ( n > 0 ){
        for ( i=n-1; i > 0; i-- ){
            if ( p[i] ==' ' || p[i] =='\t' || p[i] =='\n' || p[i] =='\r' ){
                p[i] = 0;
                continue;
            }
            else break;
        }
    }
    strcpy( str, p );
    g_free( p );
    return  str;
}

/*-----------------------------------------------------------*/
// src_job_info( ) - find job info ptr by carrier id
/*-----------------------------------------------------------*/
int     src_job_info( char *carid, TTJOB_INFO **job_info )
{
        int         i;
        TTJOB_INFO  *j_info;
    *job_info = NULL;
    for ( i=0; i < sim_job_count && sim_job_list->job_list[i] != NULL; i++ ){ // scan
        j_info = sim_job_list->job_list[i];
        if ( strcmp( carid, j_info->carid ) == 0 ){     // carid match ?
            *job_info = j_info;                         // yes, set info ptr
            return i;
        }
    }
    return NG;
}
