/****************************************************************/
/*                                                              */
/*    << DSHENG4 Test Program >>                                */
/*                                                              */
/*              file name : g_simthr.c                          */
/*                                                              */
/*  Produced by Data Map Corporation in 2009.                   */
/*  All rights reserved. No part of this program may be copied  */
/*  in anyway, or by any means, without permission from         */
/*  Data Map Corporation.                                       */
/*                                                              */
/****************************************************************/

#include    "all.h"

static  int     c_count;                                // Job Cycle Counter

/*----- sim operation state name ----------------------------*/
char    *op_name_list[] = {
    "SOPE_IDLE",               // 0
    "SOPE_LOAD_START",         // 1
    "SOPE_LOAD_END",           // 2
    "SOPE_READ_CARID",         // 3
    "SOPE_PORT_ASSOCIATED",    // 4
    "SOPE_ID_VERIFY_END",      // 5
    "SOPE_OPEN_FOUP",          // 6
    "SOPE_READ_SLOTMAP",       // 7
    "SOPE_SLOTMAP_VERIFY_END", // 8
    "SOPE_PPID_READY",         // 9
    "SOPE_PRJ_READY",          // 10
    "SOPE_CJ_READY",           // 11
    "SOPE_PROC_START",         // 12
    "SOPE_PROC_END",           // 13
    "SOPE_PRJ_COMPLETE",       // 14
    "SOPE_CJ_COMPLETE",        // 15
    "SOPE_CJ_DELETE",          // 16
    "SOPE_CLOSE_FOUP",         // 17
    "SOPE_UNLOAD_START",       // 18
    "SOPE_UNLOAD_END",         // 19
    NULL
    };
/*-----------------------------------------------------------*/
// p_thread_sim( ) - simulation thread
/*-----------------------------------------------------------*/
void    p_thread_sim( void )
{
        int     ei;
        TTJOB_INFO      *job_info;
        TTSIM_OPERATION *sim_p;
    sim_p = get_sim_operation_info( );
    sim_p->state  = SOPE_IDLE;                          // sim operation state =SOPE_IDLE
    sim_p->ope_mode  = 0;                               // step mode
    sim_p->job_index = 0;                               // from top job
    sim_p->stop_flag = 0;                               // stop flag = 0
    c_count = 0;                                        // cycle counter = 0
    Gprintf("p_thread_sim() start\n");

    while( sim_p->stop_flag==0 ){
        job_info = sim_p->job_info = get_next_job_info( sim_p->job_index, &sim_p->job_index );
        if ( job_info == NULL ) break;                  // if no job, abort
        sim_p->carid[0] = 0;
        sim_p->load_port = job_info->port;
        ei = execute_job_operation( sim_p );      // do a job simulation

        if ( ei < 0 ){
            Gprintf( "***** operation error, stopped *****\n\n" );
            break;
        }
        if ( ei == 1 ){
            Gprintf( "***** operation aborted *****\n\n" );
            break;
        }
        if ( sim_p->cycle_stop_flag != 0 ){
            Gprintf( "***** operation cycle stopped *****\n\n" );
            break;
        }
        Sleep( 1000 );
    }
    // end of simulation
    p_set_sim_busy( 0 );                          // = simulation not busy
                                                        // exit thread
}
/*-----------------------------------------------------------*/
//execute_job_operation( ) - Execute a job
/*-----------------------------------------------------------*/
int     execute_job_operation( TTSIM_OPERATION *sim_p )
{
        TTJOB_INFO  *job_info;
        int         ei = 0;
        int         end_flag = 0;                           // end of operation (one job) or error
        UINT        t;
    job_info = sim_p->job_info;

    while( end_flag==0 && sim_p->stop_flag==0 ){            // do loop bu state until end flag = 1
        switch( sim_p->state ){                             // processing by case (=sim state)
        case SOPE_IDLE:                                     // Initial state
            ei = ask_ready( );                              // wait for operator instruction
            if ( ei < 0 ) break;
            sim_p->state = SOPE_LOAD_START;                 // next state
            break;

        case SOPE_LOAD_START:
            DshStartProcessTimer(0);                        // start timer

            ei = ask_ready( );                         // wait for operator instruction
            init_load_info( );                         // clear load information
            set_load_info( );                          // set sv information

            ei = EngSetSvVal( SV_LoadPortId, &sim_p->load_port, 1 );
            if ( ei < 0 ){
                Gprintf( "!!! SV_LoadPortId set error, port id=%d\n", sim_p->load_port );
                break;
            }
            sim_p->load_transfer_status = ST_PortTrTransferBlocked; // TransferBlocked

            ei = notify_event( CE_LoadTransferBlocked );  // CE_LoadTransferBlocked
            if ( ei < 0 ) break;
            sim_p->state = SOPE_LOAD_END;
            break;

        case SOPE_LOAD_END:
            ei = ask_ready( );                         // wait for operator instruction
            if ( ei< 0 ) break;

            ei = notify_event( CE_LoadMaterialArrived );  // CE_LoadMaterialArrived
            if ( ei < 0 ) break;
            sim_p->state = SOPE_READ_CARID;
            break;

        case SOPE_READ_CARID:                               // Carrier ID read
            ei = ask_ready( );                         // wait for operator instruction
            if ( ei < 0 ) break;

            strcpy( sim_p->carid, job_info->carid );        // get carid from job_info
            ei = EngSetSvVal( SV_LoadCarId, sim_p->carid, strlen(sim_p->carid) ); // SV set
            if ( ei < 0 ) break;

            sim_p->load_carid_status      = ST_CarIdWaitingForHost; // LoadCarIdInfo
            sim_p->load_slotmap_status    = ST_SlotMapNotRead;      // LoadSlotMapStatus
            sim_p->load_car_access_status = ST_CarProcInAccess;     // LoadCarAccessStatus
            set_load_info( );                                  // set SV_xxxx
            ei = EngAllocCarInfo( sim_p->carid, &sim_p->carid_index);  // CarrierIDo^
            if ( ei < 0  ) break;

            ei = notify_event( CE_LoadWaitingForHost);// CE_LoadWaitingForHost Event
            if ( ei < 0 ) break;

            sim_p->state = SOPE_PORT_ASSOCIATED;            // sim_p->stateɐi߂B
            break;

        case SOPE_PORT_ASSOCIATED:
            sim_p->load_port_association_status = ST_PortAssociated; // Associated
            set_load_info( );

            ei = notify_event( CE_LoadAssociated );   // CE_LoadAssociated Event
            if ( ei < 0 ) break;

            sim_p->state = SOPE_ID_VERIFY_END;              // sim_p->stateɐi߂B
            break;

        case SOPE_ID_VERIFY_END:                            // S3F17M ContentMap
            ei = wait_SxFy( 3, 17, 0 );               // wait for S3F17
            if ( ei < 0 ) break;
            sim_p->sub_state = 0;
            sim_p->load_carid_status = ST_CarIdVerificationOK;
            set_load_info( );

            ei = notify_event( CE_IdVerifyOK );       // CE_IdVerifyOK
            if ( ei < 0 ) break;

            sim_p->sub_state = 1;
            ei = notify_event( CE_SubstAtSource );    // CE_SubstAtSource
            if ( ei < 0 ) break;

            sim_p->sub_state = 2;
            ei = notify_event( CE_SubstOccupied );    // CE_SubstOccupied
            if ( ei < 0 ) break;;

            sim_p->sub_state = 0;
            sim_p->state = SOPE_OPEN_FOUP;
            break;
//--------------------------------------------------------------
        case SOPE_OPEN_FOUP:                                // FOUPJ{^NbN
            ei = notify_event( CE_OpenFoup );         // CE_OpenFoup
            if ( ei < 0 ) break;
            sim_p->state = SOPE_READ_SLOTMAP;
            break;
//--------------------------------------------------------------
        case SOPE_READ_SLOTMAP:                             // Slotmapƍ{^ Click
            sim_p->load_slotmap_status = ST_SlotMapWaitingForHost;// LoadSlotMapStatus
            set_load_info( );                          // set SV

            ei = notify_event( CE_LoadWaitingForHost ); // CE_LoadWaitingForHost
            if ( ei < 0 ) break;
            sim_p->state = SOPE_SLOTMAP_VERIFY_END;
            break;

//--------------------------------------------------------------
        case SOPE_SLOTMAP_VERIFY_END:                       // S3F17 ProceedWithCarrier OK
            ei = wait_SxFy( 3, 17, 0 );
            if ( ei < 0 ) break;
            sim_p->load_slotmap_status = ST_SlotMapVerificationOK; // LoadSlotMapStatus
            set_load_info( );                          // set SV

            ei = notify_event( CE_SlotMapVerifyOK );  // CE_SlotMapVerifyOK
            if ( ei < 0 ) break;
            sim_p->state = SOPE_PPID_READY;
            break;

//--------------------------------------------------------------
        case SOPE_PPID_READY:                               // S7F3/S7F23/S15F13 PPIDM
            switch (s_get_rcp_msg() ){           // rcp msg ?
            case rcp_msg_mode_PP:                // S7F3
                ei = wait_SxFy( 7, 3, 0 );
                break;
            case rcp_msg_mode_FPP:               // S7F23
                ei = wait_SxFy( 7, 23, 0 );
                break;
            case rcp_msg_mode_RCP:               // S15F13
                ei = wait_SxFy( 15, 13, 0 );
                break;
            }
            if ( ei < 0 ) break;
            sim_p->state = SOPE_PRJ_READY;
            break;

//--------------------------------------------------------------
        case SOPE_PRJ_READY:                                // S16F11/S16F15 PrjM
            switch( s_get_prj_msg() ){                // gpMSG ?
            case prj_msg_mode_PRJ:              // S16F11
                ei = wait_SxFy( 16, 11, 0 );
                break;
            case prj_msg_mode_MPRJ:             // S16F15
                ei = wait_SxFy( 16, 15, 0 );
                break;
            }
            if ( ei < 0 ) break;
            strcpy( sim_p->prj_state, "PRJOBPOOLED" );
            ei = EngSetSvVal( SV_PrjState, sim_p->prj_state,strlen( sim_p->prj_state ) );

            ei = notify_event( CE_PrJobPooled );      // CE_PrJobPooled
            if ( ei < 0 ) break;

            sim_p->state = SOPE_CJ_READY;
            break;

//--------------------------------------------------------------
        case SOPE_CJ_READY:                                 // S14F9 ControlJob M
            sim_p->sub_state = 0;
            ei = wait_SxFy( 14, 9, 0 );               // wait for S14F9 msg
            if ( ei < 0 ) break;

            sim_p->sub_state = 1;
            sim_p->cj_state = ST_CJ_Queued;
            ei = EngSetSvVal( SV_CjState, &sim_p->cj_state, 1 );  // SV Cj_State
            ei = EngSetCjState( sim_p->cjid, sim_p->cj_state );

            ei = notify_event( CE_CJobQueued );       // CE_CJobQueued
            if ( ei < 0 ) break;

            sim_p->sub_state = 2;
            sim_p->cj_state = ST_CJ_Selected;
            ei = EngSetSvVal( SV_CjState, &sim_p->cj_state, 1 );
            ei = EngSetCjState( sim_p->cjid, sim_p->cj_state );

            ei = notify_event( CE_CJobSelected );     // CE_CJobSelected
            if ( ei < 0 ) break;

            sim_p->sub_state = 0;
            sim_p->state = SOPE_PROC_START;
            break;

//--------------------------------------------------------------
        case SOPE_PROC_START:                               // Jn{^ Click
            sim_p->sub_state = 0;
            ei = ask_ready( );                         // wait for operator instruction
            if ( ei < 0 ) break;

            sim_p->sub_state = 1;
            sim_p->cj_state = ST_CJ_Executing;              // Cj_State
            ei = EngSetSvVal( SV_CjState, &sim_p->cj_state, 1 );
            ei = EngSetCjState( sim_p->cjid, sim_p->cj_state );

            ei = notify_event( CE_ExecBegan );        // CE_ExecBegan
            if ( ei < 0 ) break;

            sim_p->sub_state = 2;
            strcpy( sim_p->prj_state, "SEETINGUP" );        // SV PrjState
            ei = EngSetSvVal( SV_PrjState, sim_p->prj_state, strlen( sim_p->prj_state ) );


            ei = notify_event( CE_PrJobSetup );       // CE_PrJobSetup
            if ( ei < 0 ) break;

            sim_p->sub_state = 3;
            sim_p->load_car_access_status = ST_CarProcInAccess; // LoadCarAccessStatus
            set_load_info( );                          // Set SV_xxxx
            ei = notify_event( CE_CarInAccess );
            if ( ei < 0 ) break;

            sim_p->sub_state = 4;
            ei = notify_event( CE_SubstAtWork );      // CE_SubstAtWork
            if ( ei < 0 ) break;

            sim_p->sub_state = 5;
            strcpy( sim_p->prj_state, "PROCESSING" );
            ei = EngSetSvVal( SV_PrjState,  sim_p->prj_state, strlen( sim_p->prj_state ) );

            ei = notify_event( CE_SubstInProcess );   // CE_SubstInProcess
            if ( ei < 0 ) break;

            sim_p->sub_state = 6;
            ei = notify_event( CE_PrJobProcessing );  // CE_PrJobProcessing
            if ( ei < 0 ) break;

            sim_p->sub_state = 0;
            sim_p->state = SOPE_PROC_END;
            break;

//--------------------------------------------------------------
        case SOPE_PROC_END:                                 // I{^ Click
            sim_p->sub_state = 0;
            ei = ask_ready( );                         // wait for operator instruction
            if ( ei < 0 ) break;

            sim_p->sub_state = 1;
            sim_p->load_car_access_status = ST_CarProcComplete; // LoadCarAccessStatus
            set_load_info( );

            ei = notify_event( CE_SubstProcessingComplete ); // CE_SubstProcessingComplete
            if ( ei < 0 ) break;

            sim_p->sub_state = 2;
            strcpy( sim_p->prj_state, "PROCCOMPLETE" );
            ei = EngSetSvVal( SV_PrjState,  sim_p->prj_state, strlen( sim_p->prj_state ) );

            ei = notify_event( CE_ProcessingComplete ); // CE_ProcessingComplete
            if ( ei < 0 )break;

            sim_p->sub_state = 3;
            ei = notify_event( CE_SubstAtDestination ); // CE_SubstAtDestination
            if ( ei < 0 )break;

            sim_p->sub_state = 0;
            sim_p->state = SOPE_PRJ_COMPLETE;               // ɐi߂B

            break;

//--------------------------------------------------------------
        case SOPE_PRJ_COMPLETE:
            ei = ask_ready( );                         // wait for operator instruction
            if ( ei < 0 ) break;

            strcpy( sim_p->prj_state, "PRJOBCOMPLETE" );
            ei = EngSetSvVal( SV_PrjState,  sim_p->prj_state, strlen( sim_p->prj_state ) );

            ei = notify_event( CE_PrJobComplete );    // CE_PrJobComplete
            if ( ei < 0 ) break;

            ei = EngDelPrjInfo( sim_p->prjid );      // PRJ폜
            sim_p->state = SOPE_CJ_COMPLETE;                // ɐi߂B
            break;

//--------------------------------------------------------------
        case SOPE_CJ_COMPLETE:
            ei = ask_ready( );                         // wait for operator instruction
            if ( ei < 0 ) break;

            ei = EngSetSvVal( SV_CjState, &sim_p->cj_state, 1 );
            sim_p->cj_state = ST_CJ_Completed;
            ei = EngSetCjState( sim_p->cjid, sim_p->cj_state );    // CJ StateXV

            ei = notify_event( CE_CJobComplete );     // CE_CJobComplete
            if ( ei < 0 ) break;

            sim_p->state = SOPE_CJ_DELETE;
            break;

//--------------------------------------------------------------
        case SOPE_CJ_DELETE:                                // S14F11 Delete Control Job
            ei = wait_SxFy( 14, 11, 0 );              // wait for S14F11
            if ( ei < 0 ) break;

            ei = EngDelCjInfo( sim_p->cjid );         // delete CJ
            sim_p->cj_state = 0;
            ei = EngSetSvVal( SV_CjState, &sim_p->cj_state, 1 );

            ei = notify_event( CE_CJobDeleted );      // CE_CJobDeleted
            if ( ei < 0 ) break;

            sim_p->state = SOPE_CLOSE_FOUP;
            break;

//--------------------------------------------------------------
        case SOPE_CLOSE_FOUP:
            sim_p->sub_state = 0;
            ei = ask_ready( );                         // wait for operator instruction
            if ( ei < 0 ) break;

            sim_p->sub_state = 1;
            ei = notify_event( CE_CloseFoup );        // CE_CloseFoup
            if ( ei < 0 ) break;

            sim_p->sub_state = 2;
            ei = notify_event( CE_CarComplete );      // CE_CarComplete
            if ( ei < 0 ) break;

            sim_p->sub_state = 3;
            ei = notify_event( CE_ReadyToUnload );    // CE_ReadyToUnload
            if ( ei < 0 ) break;

            sim_p->sub_state = 0;
            sim_p->load_transfer_status = ST_PortTrReadyToUnload;// TransferBlocked
            set_load_info( );                          // Set SV_xxxx
            sim_p->state = SOPE_UNLOAD_START;               // oJn
            break;

//--------------------------------------------------------------
        case SOPE_UNLOAD_START:                             // oI{^
            sim_p->sub_state = 0;
            ei = ask_ready( );                         // wait for operator instruction
            if ( ei < 0 ) break;

            sim_p->sub_state = 1;
            sim_p->load_transfer_status = ST_PortTrTransferBlocked; // TransferBlocked
            set_load_info( );

            ei = notify_event( CE_LoadTransferBlocked );  // CE_LoadTransferBlocked
            if ( ei < 0 ) break;

            sim_p->sub_state = 2;
            ei = notify_event( CE_MaterialRemoved );        // CE_MaterialRemoved
            if ( ei < 0 ) break;

            sim_p->sub_state = 3;
            sim_p->load_port_association_status = ST_PortNotAssociated;  // Associated
            set_load_info( );

            ei = notify_event( CE_LoadNotAssociated );      // CE_LoadNotAssociated
            if ( ei < 0 ) break;

            sim_p->sub_state = 4;
            ei = EngDelCarInfo( sim_p->carid );
//            sim_delete_substid( );                        // delete substid
            init_load_info( );

            ei = notify_event( CE_ReadyToLoad );            // CE_ReadyToLoad
            if ( ei < 0 ) break;

            sim_p->sub_state = 0;
            sim_p->proc_count++;
            sim_p->state = SOPE_IDLE;                       // end of a job
            c_count++;                                      // processing count + 1
            t = DshStopProcessTimer(0);                     // Stop Timer
            Gprintf( "\n" );
            Gprintf( "**==> # of Processing Completed = [ %d ]\n ", c_count );
            Gprintf( "      Cycle Time = %d.%03d sec\n", t/1000, t % 1000 );
            end_flag = 1;
            break;
//--------------------------------------------------------------
        default:
            break;
        }
        if ( ei < 0 ){                                      // error or abortion by operator
            Gprintf( "\n!!!!! aborted Simulation  state = %s(=%d), sub_state = %d\n\n",
                    op_name_list[sim_p->state], sim_p->state, sim_p->sub_state );
            break;
        }
    }
    return ei;
}

