/****************************************************************/
/*                                                              */
/*    << DSHENG4 Test Program >>                                */
/*                                                              */
/*              file name : g_simula.c                          */
/*                                                              */
/*  Produced by Data Map Corporation in 2009.                   */
/*  All rights reserved. No part of this program may be copied  */
/*  in anyway, or by any means, without permission from         */
/*  Data Map Corporation.                                       */
/*                                                              */
/****************************************************************/

#include    "all.h"

extern  char    *job_sche_file;                       // Job Schedule File Name

int     p_get_comm_stat( void );
void    p_thread_sim( void );

TTSIM_OPERATION *sim_ope_list;                        // Sim Operation Control Block

extern  char    *op_name_list[];                      // see g_simthr.c
/*************************************************************/
// p_SIM( ) - Simulation Start - WP scenario sample
/*************************************************************/
int     p_SIM( void )
{
        int     ei, i;
        int     stat, asize;
        char    errstr[256];
        char    buff[KEY_BUFF_SIZE];
        char    fname[256];
        TTSIM_OPERATION *sim_p;

    Gprintf( "\n" );
    Gprintf( "*======= Simulation Start =================================*\n\n" );

//----- Check if communication being established ( S1F13,14 transaction end )

    for ( i=0; i < 100; i++ ){                          // try 100*100 = 10 seconds
        stat = 0;
        ei = EngGetSvVal( SV_CommunicationState, &stat, &asize ); // get communication state
        if ( ei < 0 ){
            Gprintf( "!!!  Simulation Communicatoin status get error, aborted\n" );
            return ( -1 );
        }
        if ( stat == ST_COMMUNICATING ) break;      // SV_CommunicationState = ST_COMMUNICATING ?
        if ( p_get_comm_stat( ) == 0 ){             // GenEnable() executed ?
            p_enable( );                            // if not, do it
        }
        dsh_msleep( 10 );
    }
    if ( stat != ST_COMMUNICATING ){                // Communication  established ?
        Gprintf( "!!! Not Communicating\n" );
        return (-1);                                // Abort simulation
    }
    Gprintf( "\n" );
    Gprintf( "        Communication established\n\n" ); // Go

//----- Job Schedule File setup - jobsche.txt file ------------
//      Wafer Processing Job Schedule List (g_schedu.c)

    strcpy( fname, job_sche_file );                     // job_sche_file, see g_main.c

#if 0       // skip
    while( 1 ){
        Gprintf( "  default Job Schedule File = %s\n", fname );
                                                        // 'Enter' Key ==> default file
        input_line( "SIM> Input file name ( q=Quit, Enter=default ) : ", buff );
        if ( buff[0]=='q' || buff[0] == 'Q' ) return ( -1 );    // 'q' ==> Quit(=Abortion)
        break;
    }
    if ( buff[0] != 0 ){                                // Any file name entered ?
        strcpy( fname, buff );                          // yes, take file name entered by key
    }
#endif

    Gprintf( "         Job Schedule File = %s\n\n", fname );

    ei = setup_job_list( fname, errstr );         // Setup Job Schedule List by fname (g_schedu.c)
    if ( ei != 0 ){
        Gprintf( "!!! SIM error - %s\n", errstr );
        return ( -1 );
    }

//----- Job Schedule File OK, start simulation ----------------
    sim_p = sim_ope_list = g_calloc( sizeof( TTSIM_OPERATION ), 8300 ); // ope control blk
    s_init_msg_list( 32 );                         // setup MSG queue
    //----- Simulation processing to be done by p_thread_sim() thread
    //      Create a thread

#if ENG_OS == OS_WINDOWS
        DshCreateThread( (LPTHREAD_START_ROUTINE)p_thread_sim, NULL,  &sim_p->sim_thid );
#else
    dsh_create_thread( (LPTHREAD_START_ROUTINE)p_thread_sim, ( void* )0,  NULL, NULL, (pthread_t*)&sim_p->sim_thid );
#endif

    //----- simulation busy flag = 1 and wait for simulation end
    p_set_sim_busy( 1 );                                // make simulation busy
    while( p_get_sim_busy( ) == 1 ){                    // still busy ?
        if ( sim_p->ope_mode != 0 ){                    // if AUTO mode, then input command key here.
            input_line( "SIM> Auto mode (s=step, c=cycle stop, q=quit) : ", buff );
            if ( buff[0]=='s' || buff[0]=='S' ){
                sim_p->ope_mode = 0;                    // 's' --> op mode=step mode
            }
            else if ( buff[0]=='q' || buff[0]=='Q' ){
                sim_p->ope_mode = 0;
                sim_p->stop_flag = 1;                   // 'q' --> stop operation
            }
            else if ( buff[0]=='c' || buff[0]=='C' ){
                sim_p->cycle_stop_flag = 1;             // 'c' --> stop operation at end of the cycle
            }
        }
        dsh_msleep( 2 );
    }
    s_term_msg_list( );
//----- Simulation stopped, release allocated memory
    g_free( sim_ope_list );                          // free sim ope control blk
    free_job_list( );                              // free job schedule list
    return 0;
}

/*-----------------------------------------------------------*/
// p_set_sim_busy( ) - set sim ope busy flag
/*-----------------------------------------------------------*/
void    p_set_sim_busy( int flag )
{
    sim_ope_list->sim_busy = flag;
}
/*-----------------------------------------------------------*/
// p_get_sim_busy( ) - get sim ope busy flag
/*-----------------------------------------------------------*/
int     p_get_sim_busy( void )
{
    if ( sim_ope_list == NULL ) return 0;
    return sim_ope_list->sim_busy;
}


/*-----------------------------------------------------------*/
// init_load_info( )
/*-----------------------------------------------------------*/
void    init_load_info( void )
{
        TTSIM_OPERATION *sim_p;
    sim_p = get_sim_operation_info( );
    sim_p->carid[0] = 0;                                          // LoadCarId
    sim_p->load_transfer_status         = ST_PortTrReadyToLoad;   // TransferStatus
    sim_p->load_carid_status            = ST_CarIdNotRead;        // LoadCarIdStatus
    sim_p->load_slotmap_status          = ST_SlotMapNotRead;      // LoadSlotmapStatus
    sim_p->load_port_association_status = ST_PortNotAssociated;   // PortAssociationStatus
    sim_p->load_car_access_status       = ST_CarProcNotAccess;    // LoadCarAccessStatus
    set_load_info( );

}
/*-----------------------------------------------------------*/
// set_load_info( ) - carrier id, status
//                    load status etc
/*-----------------------------------------------------------*/
int     set_load_info( void )
{
        TTSIM_OPERATION *sim_p;
        int     ei = 0;
        int     car_index;
    sim_p = get_sim_operation_info( );
    if ( sim_p->s_print_flag ){
        Gprintf( "    transfer status    = %d\n", sim_p->load_transfer_status );
        Gprintf( "    carid status       = %d\n", sim_p->load_carid_status );
        Gprintf( "    slotmap status     = %d\n", sim_p->load_slotmap_status );
        Gprintf( "    association status = %d\n", sim_p->load_port_association_status );
        Gprintf( "    access status      = %d\n", sim_p->load_car_access_status );
    }
    if ( sim_p->carid[0] != '\0' && sim_p->state > SOPE_READ_CARID ){             // sim state > SOPE_READ_CARID ?
        ei = EngGetCarIdIndex( sim_p->carid, &car_index );                        // carid being registered ?
        if ( ei == 0 ){                                                                 // yes
            ei = EngSetCarIdStatus( sim_p->carid, sim_p->load_carid_status );     // set Carrier state
            if ( ei < 0 ){
                Gprintf( "      !!! EngSetCarIdStatus ei=%d\n", ei );
                Gprintf( "          carid = %s carid_status = %d\n",   sim_p->carid, sim_p->load_carid_status );
            }
            ei = EngSetCarMapStatus( sim_p->carid, sim_p->load_slotmap_status );
            if ( ei < 0 ){
                Gprintf( "      !!! EngSetCarMapStatus ei=%d\n", ei );
            }
            ei = EngSetCarAccessStatus( sim_p->carid, sim_p->load_car_access_status );
            if ( ei < 0 ){
                Gprintf( "      !!! EngSetCarAccessStatus ei=%d\n", ei ); ;
            }
        }
    }
    ei = EngSetSvVal( SV_LoadCarId, sim_p->carid, strlen( sim_p->carid ) );             // CarId SVXV
    if ( ei < 0 ){
        Gprintf( "      !!! EngSetSvVal(SV_LoadCarId) carid = %s ei=%d\n", sim_p->carid, ei );
    }

    ei = EngSetSvVal( SV_LoadPortTransferStatus, &sim_p->load_transfer_status, 1 );     // TransferStatus
    if ( ei < 0 ){
        Gprintf( "      !!! EngSetSvVal(SV_LoadPortTransferStatus) status = %d ei=%d\n", sim_p->load_transfer_status, ei );
    }

    ei = EngSetSvVal( SV_LoadCarIdStatus, &sim_p->load_carid_status, 1 );               // CarIdStatus
    if ( ei < 0 ){
        Gprintf( "      !!! EngSetSvVal(SV_LoadCarIdStatus) status = %d ei=%d\n", sim_p->load_carid_status, ei );
    }

    ei = EngSetSvVal( SV_LoadSlotMapStatus, &sim_p->load_slotmap_status, 1 );           // SlotmapStatus
    if ( ei < 0 ){
        Gprintf( "      !!! EngSetSvVal(SV_LoadSlotMapStatus) status = %d ei=%d\n", sim_p->load_slotmap_status, ei );
    }

    ei = EngSetSvVal( SV_LoadPortAssociationStatus, &sim_p->load_port_association_status, 1 ); // AssociationStatus
    if ( ei < 0 ){
        Gprintf( "      !!! EngSetSvVal( SV_LoadPortAssociationStatus) status = %d ei=%d\n", sim_p->load_port_association_status, ei );
    }

    ei = EngSetSvVal( SV_LoadCarAccessStatus, &sim_p->load_car_access_status, 1 );      // Car AccessStatus
    if ( ei < 0 ){
        Gprintf( "      !!! EngSetSvVal( SV_LoadCarAccessStatus) status = %d ei=%d\n", sim_p->load_car_access_status, ei );
    }
    return ei;
}

/*-----------------------------------------------------------*/
// s6f11_callback() - Callback from
/*-----------------------------------------------------------*/
static int s6f11_callback( int end_status, TCEID ceid,  ULONG upara )
{
        TTSIM_OPERATION *sim_p;
    sim_p = get_sim_operation_info( );
    if ( sim_p->ceid_callback != ceid ) return 0;           // ceid which has been sent ?
    sim_p->ce_end_status = end_status;                      // yes
    sim_p->ceid_callback = 0;                               // reset callback wait falg
    if ( end_status != 0 ){
        Gprintf( " * S6F11 response error ei=%d ceid=%d\n", end_status, ceid );
    }else{
        ; // Gprintf( "        --->> CEID=%d callback\n", ceid );
    }
    return 0;
}

/*-----------------------------------------------------------*/
// notify_event() - Send S6F11 (Collection Event)
/*-----------------------------------------------------------*/
int     notify_event( int ceid )
{
        TTSIM_OPERATION *sim_p = sim_ope_list;
        char    ce_name[81];
        int     ei;
    ei = EngGetCeName( ceid, ce_name );                     // CEID Exists ?
    if (ei < 0){
        Gprintf( "!!! Event-%d not defined", ceid );        // no
        return (-1);
    }
    sim_p->ceid_callback = ceid;                            // send ceid = callback ceid ?
    ei = EngNotifyEvent( ceid, s6f11_callback, ceid);       // send S6F11
    Gprintf( "        S6F11- %s (%d) -> state=%s\n", ce_name, ceid, op_name_list[sim_p->state], ceid, ce_name);
    if (ei >= 0){
        while( (int)sim_p->ceid_callback == ceid ){         // wait for S6F12 callback
            if ( sim_p->stop_flag == 1 ){
                Gprintf( "notify_event() ceid = %d (%s) aborted\n", ceid, ce_name );
                return (-1);                                // Quit = abortion ?
            }
            dsh_msleep( 100 );                                 // delay
        }
        if ( ( ei=sim_p->ce_end_status) != 0 ){             // ck end status ( given by callback function )
            Gprintf("         CE-%d (0x%x) %s - send error-2\n", ceid, ceid, ce_name );
        }
    }else{
        Gprintf("         CE-%d (0x%x) %s - send error-1\n", ceid, ceid, ce_name );
    }
    return ei;
}
/*------------------------------------------------------------*/
static  char    kbuf[KEY_BUFF_SIZE];

/*-----------------------------------------------------------*/
// ask_ready() - Operator Instruction - key input
//               to proceed operation step
//              ( I/O Input - Sensor signal, ID reader etc )
//     [KEY]
//          'g', "ENTER" ==> proceed a step
//          'a'          ==> auto = change into continuoust operation
//                           after then, you can make single step by entering s key
//          'q'          ==> quit, abort operation
//
/*-----------------------------------------------------------*/
int     ask_ready( void )
{
        TTSIM_OPERATION *sim_p;
        int     s;
        int     ei;
    sim_p = get_sim_operation_info( );
    s = sim_p->state;
    while( 1 ){
        Gprintf( "\n" );
        Gprintf( "*SIM state = %d(%s)\n", s, op_name_list[s] );
        if ( sim_p->ope_mode == 1 ) return 0;
        input_line( "SIM> ready to go (a=auto g=go, q=quit) : ", kbuf );
        s_trim( kbuf );
        if ( kbuf[0] == 'a' || kbuf[0]=='A' ){ ei=0; sim_p->ope_mode = 1; break; }  // auto
        if ( kbuf[0] == 'g' || kbuf[0]=='G' ){ ei=0; break; }                       // step
        if ( kbuf[0] == 'q' || kbuf[0]=='Q' ){ ei=(-1); break; }                    // quit( abort )
        if ( kbuf[0] == 0 ){ ei=0; break; }                                         // step
        Gprintf( " ???, try again\n" );
    }
    return ei;
}

#define     TIMEOUT_SxFy        12000               // 2min * 60sec * 100 = 12000
/*-----------------------------------------------------------*/
// wait_SxFy() - wait for SxFy from host
//               primary message S3F17, S7F23 etc
/*-----------------------------------------------------------*/
int     wait_SxFy( int s, int f, int option )
{
        TTSIM_OPERATION *sim_p;
        int             st;
        TMSG_QUEUE_INFO *qmsg;
        DSHMSG          *minfo;
        int             sxf;
        int             ei;
        int             n = 0;
    sim_p = get_sim_operation_info( );
    st = sim_p->state;
    Gprintf( "\n" );
    Gprintf( "*SIM state = %d(%s)\n", st, op_name_list[st] );
    Gprintf( "   - wait_SxFy() s%df%d opt=%d\n", s, f, option );
    while( sim_p->stop_flag != 1 && n < TIMEOUT_SxFy ){
        if ( s_get_msg_list( &qmsg )== 0 ){           // g_poll.cmsg_listɗ^ĂB
            dsh_msleep( 1000 );
            n++;
        }else{
            minfo = qmsg->smsg;
            if ( (int)minfo->stream == s && (int)minfo->function == f ){ // ү=msg ?
                sxf = s * 256 + f;
                switch( sxf ){
/*-----------------------------------------------------------*/
                case 3*256 + 17:
                    ei = sim_s3f17_proc( qmsg, sim_p->carid, option );
                    break;
/*-----------------------------------------------------------*/
                case 7 * 256 + 3:
                    ei = sim_s7f3_proc( qmsg );
                    break;
/*-----------------------------------------------------------*/
                case 7 * 256 + 23:
                    ei = sim_s7f23_proc( qmsg );
                    break;
/*-----------------------------------------------------------*/
                case 15*256 + 13:
                    ei = sim_s15f13_proc( qmsg );
                    break;
/*-----------------------------------------------------------*/
                case 16*256 + 11:
                    ei = sim_s16f11_proc( qmsg );
                    break;
/*-----------------------------------------------------------*/
                case 16*256 + 15:
                    ei = sim_s16f15_proc( qmsg );
                    break;
/*-----------------------------------------------------------*/
                case 14*256 + 9:
                    ei = sim_s14f9_proc( qmsg );
                    break;
/*-----------------------------------------------------------*/
                case 14*256 + 11:
                    ei = sim_s14f11_proc( qmsg );
                    break;
                }
                DshFreeMsg( minfo );                            // Free smsg
                g_free( qmsg );                                 // free qmsg
                if ( ei == 0 || ei == (-1) ) return ei;
            }else{
                s_proc_rcv_msg( qmsg );                         // msgłȂmsg̏ ==> g_poll.c
                DshFreeMsg( minfo );                            // Free smsg
                g_free( qmsg );                                   // free qmsg
            }
        }
    }
    if ( n >= TIMEOUT_SxFy ) Gprintf( "!! wait_SxFy() Timeout - aborted - s%df%d \n", s, f );
    else Gprintf( "!! wait_sxfy() aborted by operator - s%df%d \n", s, f );
    return (-1);
}
/*-----------------------------------------------------------*/
// get_sim_operation_info() - get operation control blk ptr
/*-----------------------------------------------------------*/
TTSIM_OPERATION* get_sim_operation_info( void )
{
    if ( sim_ope_list == NULL ) return NULL;
    return sim_ope_list;
}
/*-----------------------------------------------------------*/
// get_sim_operation_busy() - get sim operation busy flag
/*-----------------------------------------------------------*/
int     get_sim_operation_busy( void )
{
        TTSIM_OPERATION *sim_p;
    sim_p = get_sim_operation_info( );
    if ( sim_p == NULL ) return 0;
    else return 1;
}

/*-----------------------------------------------------------*/
// get_sim_state() - get sim operation state
/*-----------------------------------------------------------*/
int     get_sim_state( void )
{
        TTSIM_OPERATION *sim_p;
    sim_p = get_sim_operation_info( );
    if ( sim_p == NULL ) return (-1);
    return sim_p->state;
}
/*-----------------------------------------------------------*/
// sim_ope_mode() - ONLINE(=1), OFFLINE(=0)
//                  S1F17, S1F15
/*-----------------------------------------------------------*/
void    sim_ope_mode( int mode )
{
        TTSIM_OPERATION *sim_p;
    sim_p = get_sim_operation_info( );
    if ( sim_p == NULL ) return;
    sim_p->operation_mode = mode;
}
/*-----------------------------------------------------------*/
// sim_access_mode() - S3F27 Change Access
/*-----------------------------------------------------------*/
void    sim_access_mode( int mode )
{
        TTSIM_OPERATION *sim_p;
    sim_p = get_sim_operation_info( );
    if ( sim_p == NULL ) return;
    sim_p->acess_mode = mode;
}
/*-----------------------------------------------------------*/
// sim_set_ppid() - S7F23 process program id
/*-----------------------------------------------------------*/
void    sim_set_ppid( char *ppid )
{
        TTSIM_OPERATION *sim_p;
    sim_p = get_sim_operation_info( );
    if ( sim_p == NULL ) return;
    strcpy( sim_p->ppid, ppid );
    Gprintf( "        Process Program id = %s\n", sim_p->ppid );
}
/*-----------------------------------------------------------*/
// sim_set_cjid() - S14F9 control job id
/*-----------------------------------------------------------*/
void    sim_set_cjid( char *cjid )
{
        TTSIM_OPERATION *sim_p;
    sim_p = get_sim_operation_info( );
    if ( sim_p == NULL ) return;
    strcpy( sim_p->cjid, cjid );
    Gprintf( "        Control Job id = %s\n", sim_p->cjid );
}
/*-----------------------------------------------------------*/
// sim_set_prjid() - S16F11 or S16F15 process job ID
/*-----------------------------------------------------------*/
void    sim_set_prjid( char *prjid )
{
        TTSIM_OPERATION *sim_p;
    sim_p = get_sim_operation_info( );
    if ( sim_p == NULL ) return;
    strcpy( sim_p->prjid, prjid );
    Gprintf( "        Process Job id = %s\n", sim_p->prjid );
}

/*------------------------------------------------------------*/
// get_next_job_info() - get next job info pointer
/*------------------------------------------------------------*/
TTJOB_INFO  *get_next_job_info( int cur_index, int *next_index )
{
        TTSIM_JOB_LIST  *s_list;
        TTJOB_INFO      *info;
    s_list = get_sim_list_ptr( );
    if ( cur_index >= s_list->job_count ) cur_index = 0;
    info = s_list->job_list[cur_index];
    *next_index = cur_index+1;
    s_printf_job_info( cur_index, info );
    return info;
}

/*------------------------------------------------------------*/
// s_print_job_info() - print job info
/*------------------------------------------------------------*/
void    s_printf_job_info( int index, TTJOB_INFO *info )
{
    Gprintf( "\n" );
    Gprintf( "===== JOB List - No. %d ===============================\n", index );
    Gprintf( "        port   = %d\n", info->port );
    Gprintf( "        carid  = %s\n", info->carid );

#if 0
        char    *carid;             // carrier id
        int     port;               // port 1,2
        char    *usage;             // usage
        TTSLOT_INFO *slot_info[K_MAX_SLOT]; // slot information
        char    *cjid;              // control job id
        char    *prjid;             // prjid
        char    *rcpid;             // rcpid( = ppid )
        char    *rcpbody;           // rcp body( = ppbody, ccode )
        int     para_count;         // # of parameter
        int     para_index;         // next to put
        TTRCP_PARA *rcp_para[K_MAX_PARA];   // parameter
#endif

}
/*============================================================*/

static  TDshList    *msg_q_list;
/*------------------------------------------------------------*/
// s_init_msg_list() - create msg queue list for simulation
/*------------------------------------------------------------*/
void    s_init_msg_list( int size )
{
    msg_q_list = g_calloc( sizeof(TDshList), 8301 );
    DshInitList( msg_q_list, size, 10000 );
}
/*------------------------------------------------------------*/
// s_term_msg_list() - terminate msg queue list for simulation
/*------------------------------------------------------------*/
void    s_term_msg_list( void )
{
        TMSG_QUEUE_INFO *qmsg;
    if ( msg_q_list == NULL ) return;
    while( ( s_get_msg_list( &qmsg)) != 0 ){        // make queue empty
        DshFreeMsg( qmsg->smsg );                   // Free smsg
    }
    DshTermList( msg_q_list );
    g_free( msg_q_list );
    msg_q_list = NULL;
}
/*------------------------------------------------------------*/
// s_put_msg_list() - put msg on the queue list for simulation
/*------------------------------------------------------------*/
int     s_put_msg_list( TMSG_QUEUE_INFO *qmsg )
{
    if ( msg_q_list == NULL ) return NG;
    return DshPutList( msg_q_list, (void*)qmsg );
}
/*------------------------------------------------------------*/
// s_get_msg_list() - get msg fro the queue list for simulation
/*------------------------------------------------------------*/
int     s_get_msg_list( TMSG_QUEUE_INFO **qmsg )
{
    if ( msg_q_list == NULL ) return 0;
    return DshGetList( msg_q_list, (void**)qmsg );
}
